
/* -----                                                                */
/*                                                                      */
/* UWDIALER Version 1.1                                        03/01/95 */
/*                                                                      */
/* AUTHORS: William C. Brown and Gerry A. Graesser                      */
/* VERSION: 1.1 (beta)                                                  */
/* DESCRIPTION: Remove those pesky lock files that undial() doesn't     */
/*		want to remove for me...				*/
/*                                                                      */
/*                                                                ----- */

#include <stdio.h>
#include <dirent.h>
#include <termio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/OpenLook.h>
#include <Xol/ScrollingL.h>
#include "uwdialer.h"

pid_t GetLockPid(char *Path)
{
	pid_t	lock_pid;
	FILE	*fp, *fopen();
	char	Buffer[BUFSIZ];

	if((fp = fopen(Path, READ)) == NULL) {
		(void)DialerError(NONFATAL, CANTOPENLOCK, Path);
		lock_pid = 0;
	} else {

		/* -----					*/
		/* Should only need one line to get the pid out */
		/*					  ----- */

		(void)fgets(Buffer, BUFSIZ, fp);
		(void)fclose(fp);

		lock_pid = (pid_t)atol(Buffer);
	}

	return(lock_pid);

} /* GetLockPid */

void Rewrite_Lock(char *Path, pid_t NewPid)
{
	char	TmpLockFile[BUFSIZ];
	char	AsciiPid[12];
	struct stat	LkStatBuf;
	int	Fd;

	if (stat(Path, &LkStatBuf) != 0) {
		(void)DialerError(NONFATAL, STATLOCKFILE, Path, errno);
		return;
	}

	/*
	 * Use the current PID for the name of the temporary lock file
	 * to avoid potential race conditions between child and parent
	 */
	(void)sprintf(AsciiPid, "%10ld\n", NewPid);
	(void)sprintf(TmpLockFile, "/var/spool/locks/LTMP.%ld", getpid());

	if ((Fd = creat(TmpLockFile, 0644)) < 0) {
		(void)DialerError(NONFATAL, CREATTMPLOCK, TmpLockFile, errno);
		return;
	}

	if (write(Fd, AsciiPid, 11) != 11) {
		(void)DialerError(NONFATAL, WRITETMPLOCK, TmpLockFile, errno);
		(void)close(Fd);
		(void)unlink(TmpLockFile);
		return;
	}

	if (chmod(TmpLockFile, LkStatBuf.st_mode) != 0) {
		(void)DialerError(NONFATAL, CHMODTMPLOCK, TmpLockFile, errno);
		(void)close(Fd);
		(void)unlink(TmpLockFile);
	}

	if (chown(TmpLockFile, LkStatBuf.st_uid, LkStatBuf.st_gid) != 0) {
		(void)DialerError(NONFATAL, CHOWNTMPLOCK, TmpLockFile, errno);
		(void)close(Fd);
		(void)unlink(TmpLockFile);
		return;
	}

	if (close(Fd) != 0) {
		(void)DialerError(NONFATAL, CLOSETMPLOCK, TmpLockFile, errno);
	}

	if (rename(TmpLockFile, Path) != 0) {
		(void)DialerError(NONFATAL, RENAMTMPLOCK, Path, errno);
		(void)unlink(TmpLockFile);
	}

} /* Rewrite */

void Delete_Lock(int Action, pid_t LockPid, pid_t NewPid)
{
	DIR	*dirp;
	struct dirent *dirbuf;
	char	Path[BUFSIZ];
	pid_t	lock_pid;

	if((dirp = opendir("/var/spool/locks")) == NULL) {
		(void)DialerError(NONFATAL, CANTOPENLOCKDIR);
		return;
    	}

	while((dirbuf = readdir(dirp)) != NULL) {
        	if(*dirbuf->d_name == '.')
            		continue;

		(void)sprintf(Path, "/var/spool/locks/%s", dirbuf->d_name);

		lock_pid = GetLockPid(Path);

		if(Action == REWRITE) {
			if(lock_pid == LockPid) {
				(void)Rewrite_Lock(Path, NewPid);
				break;
			} else {
				if(lock_pid == NewPid)
					break;
				/* already updated [by child] */
			}
		} else {
			if((lock_pid == LockPid) ||
			  ((lock_pid == NewPid) && (lock_pid != 0))) {
				(void)unlink(Path);
				break;
			}
		}
        }
	(void)closedir(dirp);

} /* Remove_Lock */
