
/* -----                                                                */
/*                                                                      */
/* UWDIALER Version 1.1                                        03/01/95 */
/*                                                                      */
/* AUTHORS: William C. Brown and Gerry A. Graesser                      */
/* VERSION: 1.1 (beta)                                                  */
/* DESCRIPTION: Supporting functions.                                   */
/*									*/
/*                                                                ----- */

#include <stdio.h>
#include <stropts.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mkdev.h>
#include <signal.h>
#include <ctype.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/OpenLook.h>
#include <Xol/ScrollingL.h>
#include <varargs.h>
#include <stropts.h>
#include <termio.h>
#include <dial.h>
#include "uwdialer.h"

static int Button_Busy[DIAL_BUTTONS];

void Call_Quit(Widget W, XtPointer ClientData, XtPointer CallData)
{
	struct termios t;
	extern int Modem_fd;
	extern XtInputId Modem_Input;

	if(Modem_fd != 0) {
		(void)Call_Hangup(NULL, NULL, NULL);
		undial(Modem_fd);
	}
		
        exit(0);
} /* Call_Quit */

void Call_Hangup(Widget W, XtPointer ClientData, XtPointer CallData)
{
	struct termios t;
	extern pid_t Lock_Pid, Server_Pid, Xfer_Pid;
	extern int Modem_fd;
	extern int LOGGER;
	extern int Xfer_Direction;
	extern int Xfer_Kill;
	extern FILE *Logger_fp;
	extern char Logger_Char;
	extern Widget LoggerTop;
	extern Widget XferTop;
	extern XtInputId Modem_Input;

	if(Modem_fd != 0) {
		if(Modem_Input != NULL) {
			XtRemoveInput(Modem_Input);
			Modem_Input = NULL;
		}
		tcgetattr(Modem_fd, &t);
		t.c_cflag = 0;
		t.c_lflag = 0;
		t.c_iflag = 0;
		t.c_oflag = 0;
		tcsetattr(Modem_fd, TCSANOW, &t);
		undial(Modem_fd);
		close(Modem_fd);
		Modem_fd = 0;
		Call_Record(0);	/* normal disconnect */
	}

	if(Server_Pid != 0) {
		if(kill(Server_Pid, 0) == 0)
			(void)kill(Server_Pid, SIGKILL);
		Call_Record(INTRPT);
	}

	if (LoggerTop != NULL) {
		XtDestroyWidget(LoggerTop);
		LoggerTop = NULL;
	}
	LOGGER = OFF;
	if (Logger_fp != NULL) {
		if(Logger_Char) {
			(void)write(fileno(Logger_fp), "\n", 1);
			Logger_Char = 0;
		}
		(void)fclose(Logger_fp);
		Logger_fp = NULL;
		(void)Update_Display_Logger();
	}

	if(XferTop != NULL) {
		XtDestroyWidget(XferTop);
		XferTop = NULL;
		if(Xfer_Pid == 0) Xfer_Direction = 0;
	}
	if(Xfer_Pid != 0) {
		(void)kill(-Xfer_Pid, SIGKILL);
		Xfer_Kill = SIGKILL;
	}

	(void)Delete_Lock(DELETE, Lock_Pid, Server_Pid);
	(void)Set_Buttons(SET_FOCUS, True);
	(void)Set_Buttons(SET_HANGUP, True);
	(void)Set_Buttons(SET_BREAK, True);
	(void)Set_Buttons(SET_LOG, True);
	(void)Set_Buttons(SET_UPLOAD, True);
	(void)Set_Buttons(SET_DOWNLOAD, True);
	(void)Set_Buttons(SET_TERMINATE, True);
	if(InitModemKey == 0) (void)Set_Buttons(SET_MODEM, True);
	(void)Display_Msg(False, "Ready...");
	(void)Update_Connect_Clock((XtPointer)RESET, NULL);
	(void)Set_Labels(INITIAL_SET);
	(void)Set_Buttons(SET_QUIT, False);
	(void)Set_Buttons(SET_SYSTEMS, False);
	(void)Set_Buttons(SET_MANUAL, False);
	if(InitModemKey) (void)Set_Buttons(SET_MODEM, False);
	(void)Set_Buttons(SET_HELP, False);
} /* Call_Hangup */

void Call_Modem(Widget W, XtPointer ClientData, XtPointer CallData)
{
	extern	int Modem_fd;
	extern	pid_t Server_Pid;
	extern	Widget Stub;
	extern	Dial_Info DialInfoBuf;
	extern	char ManTtyLine[];
	extern	char InitModemLine[];
	static	char Temp[TTY_SYS_NAME_MAX+9];
	struct	stat statbuf;
	major_t	majdevno;
	major_t	majrdevno;
	minor_t	minrdevno;
	int	len;
	char	oldchar;


	if((Modem_fd != 0) || (Server_Pid != 0)) {
		(void)DialerError(NONFATAL, ALREADYCONNECT);
		return;
	}

	if(InitModemKey == 0) {
		(void)DialerError(NONFATAL, NOMODEMKEY);
		return;
	}

	if(InitModemLine[0] == 0) {
		(void)DialerError(NONFATAL, NOMODEMTTY);
		if(ManTtyLine[0] == 0) return;
		(void)strcpy(InitModemLine, ManTtyLine);
	}

	if(strncmp(InitModemLine, "/dev/", 5) == 0)
		(void)strcpy(Temp, InitModemLine);
	else
		(void)sprintf(Temp, "/dev/%s", InitModemLine);

	/* Verify port and get Major/Minor device numbers of the TTY to be initialized */
	if((stat(Temp, &statbuf) != 0) ||
	  ((statbuf.st_mode & S_IFMT) != S_IFCHR)) {
		InitModemLine[0] = 0;
		(void)DialerError(NONFATAL, NOTTYDEV, Temp);
		return;
	}
	
	if (InitModemKey != (char)NULLMODEMKEY) {

		majdevno = major(statbuf.st_dev);
		majrdevno = major(statbuf.st_rdev);
		minrdevno = minor(statbuf.st_rdev);

		len = strlen(Temp);

		/* Append the key to the name of the current device */
		Temp[len] = InitModemKey;
		Temp[++len] = 0;

		if((stat(Temp, &statbuf) != 0) ||
		  ((statbuf.st_mode & S_IFMT) != S_IFCHR) ||
		   (major(statbuf.st_dev) != majdevno) ||
		   (major(statbuf.st_rdev) != majrdevno) ||
		   (minor(statbuf.st_rdev) != minrdevno)) {

			/* Try replacing the last [h/s] character */
			Temp[--len] = 0;
			oldchar = Temp[len-1];	/* save in case of error */
			Temp[len-1] = InitModemKey;

			if((stat(Temp, &statbuf) != 0) ||
			  ((statbuf.st_mode & S_IFMT) != S_IFCHR) ||
			   (major(statbuf.st_dev) != majdevno) ||
			   (major(statbuf.st_rdev) != majrdevno) ||
			   (minor(statbuf.st_rdev) != minrdevno)) {

				Temp[len-1] = oldchar;
				(void)DialerError(NONFATAL, NOMODEMDEV, &Temp[5], InitModemKey);
				InitModemLine[0] = 0;
				return;
			}
		}
	}

	(void)Set_Buttons(SET_MODEM, True);
	(void)Set_Buttons(SET_MANUAL, True);
	(void)Set_Buttons(SET_SYSTEMS, True);

	(void)memset(&DialInfoBuf, NULL, sizeof(Dial_Info));

	DialInfoBuf.Line = &Temp[5];	/* skip over /dev/ */

	XSync(XtDisplay(Stub), False);
	(void)Init_Dial(&DialInfoBuf, MODEM);
}

void Call_Manual(Widget W, XtPointer ClientData, XtPointer CallData)
{
	(void)Set_Buttons(SET_SYSTEMS, True);
	(void)Set_Buttons(SET_MANUAL, True);
	(void)Set_Buttons(SET_MODEM, True);
	(void)Settings(MANUAL);
}

void Call_Upload(Widget W, XtPointer ClientData, XtPointer CallData)
{
	Xfer_Popup(UPLOAD);
}

void Call_Downld(Widget W, XtPointer ClientData, XtPointer CallData)
{
	Xfer_Popup(DOWNLOAD);
}

void Call_SndBrk(Widget W, XtPointer ClientData, XtPointer CallData)
{
	extern int Modem_fd;
	extern int Xtermid;

	if (Modem_fd == 0) {
		(void)DialerError(NONFATAL, NOTCONNECTED);
	} else {
		(void)tcsendbreak(Modem_fd, 0);
	}
	(void)Call_Enter(NULL, (XtPointer)Xtermid, NULL, NULL);
}

void Call_Terminate(Widget W, XtPointer ClientData, XtPointer CallData)
{
	extern pid_t Xfer_Pid;
	extern int Xfer_Kill;
	extern int Xtermid;

	if(Xfer_Pid == 0) {
		(void)DialerError(NONFATAL, NOXFERCHILD);
	} else {
		(void)kill(-Xfer_Pid, SIGTERM);
		Xfer_Kill = SIGTERM;
	}
	(void)Call_Enter(NULL, (XtPointer)Xtermid, NULL, NULL);
}

void Call_Log(Widget W, XtPointer ClientData, XtPointer CallData)
{
	(void)Logger_Popup();
} /* Call_Log */

XtPointer CONVERT OLARGLIST((w, from_string, to_type, to_size, success))
OLARG(Widget, w)
OLARG(char*, from_string)
OLARG(char*, to_type)
OLARG(int, to_size)
OLGRA(Boolean*, success)
{
    XrmValue		fromVal, toVal;	/* resource holders		*/
    Boolean		convResult;	/* return value			*/
    unsigned char	oByte;		/* one byte result		*/
    unsigned short	tByte;		/* two byte result		*/
    caddr_t		fByte;		/* four byte result		*/
    caddr_t		aByte;		/* allocated result		*/
    
    fByte = aByte = NULL;
    *success = False;

    if( strcmp(XtRString, to_type) == 0 )
    {
	*success = True;
	return( from_string );
    }

    to_size = strlen(from_string);
        
    fromVal.size = strlen(from_string) + 1;
    fromVal.addr = from_string;

    switch( to_size )
    {
    case 1:
	toVal.size = sizeof(unsigned char);
	toVal.addr = (caddr_t)&oByte;
	break;
    case 2:
	toVal.size = sizeof(unsigned short);
	toVal.addr = (caddr_t)&tByte;
	break;
    default:
	toVal.size = sizeof(caddr_t);
	toVal.addr = (caddr_t)&fByte;
	break;
    }
    
    convResult = XtConvertAndStore(w, 
				   XtRString, 
				   &fromVal,
				   to_type,
				   &toVal);
    
    if( convResult )
    {
	switch( to_size )
	{
	case 1:
	    fByte = (caddr_t)oByte;
	    break;
	case 2:
	    fByte = (caddr_t)tByte;
	    break;
	default:
	    break;
	}
    }

    if( !convResult && toVal.size != to_size )
    {
	toVal.addr = (caddr_t)XtMalloc(toVal.size);
	fByte = aByte = toVal.addr;
	convResult = XtConvertAndStore(w, 
				       XtRString, 
				       &fromVal,
				       to_type,
				       &toVal);
    }
    
    if( !convResult )
    {
	XtFree(aByte);
	aByte = NULL;
    }
    
    *success = convResult;
    return( fByte );
}

Pixel GetPixelByName(Widget w, char *colorname)
{
	Display *dpy = XtDisplay(w);
	Colormap cmap = DefaultColormap(dpy, DefaultScreen(dpy));
	XColor color, ignore;

	if(XAllocNamedColor(dpy, cmap, colorname, &color, &ignore))
		return(color.pixel);
	else 
		(void)DialerError(NONFATAL, COLORALLOCATION, colorname);
}

void DialerError(Fatal, va_alist)
int Fatal;
va_dcl
{
	FILE *fopen(), *fp;
	va_list ap;
	char *ptr;
	Boolean NoConsole;

	if((fp = fopen("/dev/conslog", "w")) == NULL)
		NoConsole = True;

	va_start(ap);
	ptr = va_arg(ap, char *);

	(void)fprintf(stderr, "UWDIALER ERROR: ");
	(void)vfprintf(stderr, ptr, ap);

	if(NoConsole == False) {
		(void)fprintf(fp, "UWDIALER ERROR: ");
		(void)vfprintf(fp, ptr, ap);
		(void)fclose(fp);
	}

	va_end(ap);

	if(Fatal == FATAL)
		exit(0);
}

struct termios Tc_New;

void Init_Xterm()
{
	extern struct termios old;
	extern XtAppContext app_con;
	extern int Modem_fd;
	extern XtInputId Term_Input;
	
	tcgetattr(0, &old);
        (void)memcpy(&Tc_New, &old, sizeof(struct termios));

	/*
	 * Make sure input/ouput speed is the same
	 * (i.e., don't leave ispeed at 300 baud)
	 */
	cfsetispeed(&Tc_New, cfgetospeed(&Tc_New));

        /* Run our terminal in RAW() mode */

        Tc_New.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP| IXON);
        Tc_New.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
        Tc_New.c_cflag &= ~(CSIZE | PARENB);
        Tc_New.c_oflag &= ~(OPOST);
        Tc_New.c_cc[VMIN] = 1;
        Tc_New.c_cc[VTIME] = 0;
        tcsetattr(0, TCSANOW, &Tc_New);

        Term_Input = XtAppAddInput(app_con, Modem_fd, XtInputReadMask,
                         Chars_At_Term, NULL);

} /* Init_Xterm */

void Chars_At_Term(XtPointer ClientData, int *Src, XtInputId *inid)
{
        char buffer[BUFSIZ];
        int nbytes;
	extern int Modem_fd;
	extern int Master_fd;
	extern pid_t Xfer_Pid;
	extern int SendTo;

        if((nbytes = read(fileno(stdin), buffer, BUFSIZ)) < 0) {
                exit(0);
        } else {
		if(Xfer_Pid == 0) {
			(void)write(Modem_fd, buffer, nbytes);
		} else {
			if(SendTo == LOCALCOMMAND)
                		(void)write(Master_fd, buffer, nbytes);
		}
	}
} /* Chars_At_Term */

void Set_Buttons(enum dial_buttons Button, int Indicator)
{
	Arg args[1];
	int Sentinel;
	extern Widget Flats;
	extern enum button_sets Button_Set;

	/* search for this button in the current set */
	for(Sentinel=0; Sentinel<MAIN_BUTTONS; Sentinel++) {
		if(Button_Sets[Button_Set][Sentinel] != Button) continue;
		XtSetArg(args[0], XtNbusy, Indicator);
        	OlFlatSetValues(Flats, Sentinel, args, 1);
	}
	/* Save the Button Busy Indicator */
	Button_Busy[Button] = Indicator;
} /* Set_Buttons */

void Set_Labels(enum button_sets New_Set)
{
	Arg args[1];
	int Sentinel;
	enum dial_buttons Button;
	extern Widget Flats;
	extern enum button_sets Button_Set;

	Button_Set = New_Set;

	for(Sentinel=0; Sentinel<MAIN_BUTTONS; Sentinel++, Button++) {
		Button = Button_Sets[New_Set][Sentinel];
		XtSetArg(args[0], XtNlabel, Button_Label[Button]);
		OlFlatSetValues(Flats, Sentinel, args, 1);
		XtSetArg(args[0], XtNbusy, Button_Busy[Button]);
		OlFlatSetValues(Flats, Sentinel, args, 1);
	}

} /* Set_Labels */
