
/* -----                                                                */
/*                                                                      */
/* UWDIALER Version 1.1                                        03/01/95 */
/*                                                                      */
/* AUTHORS: William C. Brown and Gerry A. Graesser                      */
/* VERSION: 1.1 (beta)                                                  */
/* DESCRIPTION: Display messages in the status area.                    */
/*									*/
/*                                                                ----- */

#include <stdio.h>
#include <stropts.h>
#include <termio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/OpenLook.h>
#include <Xol/Stub.h>
#include <Xol/ScrollingL.h>
#include "uwdialer.h"

static char Previous_Msg[BUFSIZ];
static char Previous_Time[BUFSIZ];

enum xfer_msgs Xfer_Message;

const int ConStubHeight = 48;	/* keep this a multiple of 8 */
const int StatMsgYdelta = 30;	/* keep this 5/8 of ConStubHeight */
const int ClockYdelta = 30;	/* keep this 5/8 of ConStubHeight */
const int LoggerYdelta = 42;	/* keep this 7/8 of ConStubHeight */
const int XferYdelta = 42;	/* keep this 7/8 of ConStubHeight */

const int ClockXoffset = 5;
const int StatMsgXoffset = 100;
const int LoggerXoffset = 100;
const int XferXoffset = 150;

const char *Xfer_Msgs[XFER_MSGS] = {
	"",				/* NULL_MSG	  */
	"- UPLOAD IN PROGRESS -",	/* UPLOAD_MSG	  */
	"- DOWNLOAD IN PROGRESS -",	/* DOWNLOAD_MSG	  */
	"- LOCAL UPLOAD COMMAND -",	/* LOCAL_UP_MSG	  */
	"- LOCAL DOWNLOAD COMMAND -",	/* LOCAL_DOWN_MSG */
	"- LOCAL COMMAND MODE -",	/* LOCAL_CMD_MSG  */
};

const char *Logger_Msg = "- LOG -";

void Display_Msg(Boolean Init, char *Msg)
{
	extern Widget Stub, ConStub;
	extern GC XORgc;

	(void)Install_Font(SMALL, XORgc);

	if((Boolean)Init == True) {
		(void)memset(Previous_Msg, NULL, BUFSIZ);
		(void)strcpy(Previous_Msg, Msg);
	}
	else {
		XDrawString(XtDisplay(Stub), XtWindow(ConStub), XORgc,
			StatMsgXoffset, StatMsgYdelta,
			Previous_Msg, strlen(Previous_Msg));
		(void)memset(Previous_Msg, NULL, BUFSIZ);
		(void)strcpy(Previous_Msg, Msg);
	}

	XDrawString(XtDisplay(Stub), XtWindow(ConStub), XORgc,
		 StatMsgXoffset, StatMsgYdelta, Msg, strlen(Msg));

	XSync(XtDisplay(Stub), False);

} /* Display_Msg */

void Update_Connect_Clock(XtPointer Ind, XtIntervalId *Id)
{
	static int Hours;
	static int Mins;
	char Time[10];

	extern Widget Stub, ConStub;
	extern GC XORgc;
	extern XtAppContext app_con;
	extern XtIntervalId TimerId; 

	(void)Install_Font(LARGE, XORgc);

	if((int)Ind == INIT) {
		(void)memset(Previous_Time, NULL, BUFSIZ);
		Hours = 0;
		Mins  = 0;
	}

	if((int)Ind == RESET) {
		XtRemoveTimeOut(TimerId);
		XDrawString(XtDisplay(Stub), XtWindow(ConStub), XORgc,
			 ClockXoffset, ClockYdelta,
			 Previous_Time, strlen(Previous_Time));
		(void)Update_Connect_Clock((XtPointer)INIT, NULL);
		return;
	}

	if(((int)Ind == START) || ((int)Ind == RUN)) {
		XDrawString(XtDisplay(Stub), XtWindow(ConStub), XORgc,
			 ClockXoffset, ClockYdelta,
			 Previous_Time, strlen(Previous_Time));
		if((int)Ind == RUN) {
			Mins++;
			if((Mins % 60) == 0) {
				Mins = 0;
				Hours++;
			}
		}
	}

	if((Hours < 10) && (Mins < 10))
		(void)sprintf(Time, "0%d:0%d", Hours, Mins);
	else if ((Hours >= 10) && (Mins < 10))
		(void)sprintf(Time, "%d:0%d", Hours, Mins);
	else if ((Hours < 10) && (Mins >= 10))
		(void)sprintf(Time, "0%d:%d", Hours, Mins);
	else
		(void)sprintf(Time, "%d:%d", Hours, Mins);

	XDrawString(XtDisplay(Stub), XtWindow(ConStub), XORgc,
		 ClockXoffset, ClockYdelta, Time, strlen(Time));

	if(((int)Ind == START) || ((int)Ind == RUN)) {
		TimerId = XtAppAddTimeOut(app_con, 60000,
					  Update_Connect_Clock, RUN);
	}

	(void)strcpy(Previous_Time, Time);
	XSync(XtDisplay(Stub), False);

} /* Update_Connect_Clock */

void Call_Repaint_Display(Widget W, XEvent *xevent, Region region)
{
	extern Widget Stub, ConStub;
	extern XGCValues gcv;
	extern GC ORgc, XORgc;
	extern Pixel RedPixel, GreenPixel;
	extern int LOGGER;
	extern pid_t Xfer_Pid;
	extern int SendTo;

	gcv.foreground = GreenPixel;
	XChangeGC(XtDisplay(Stub), ORgc, GCForeground, &gcv);

	(void)Install_Font(LARGE, ORgc);
	XDrawString(XtDisplay(Stub), XtWindow(ConStub), ORgc,
		 ClockXoffset, ClockYdelta,
		 Previous_Time, strlen(Previous_Time));

	(void)Install_Font(SMALL, ORgc);
	XDrawString(XtDisplay(Stub), XtWindow(ConStub), ORgc,
		 StatMsgXoffset, StatMsgYdelta,
		 Previous_Msg, strlen(Previous_Msg));

	if(LOGGER == ON) {
		gcv.foreground = RedPixel;
		XChangeGC(XtDisplay(Stub), ORgc, GCForeground, &gcv);
		(void)Install_Font(TINY, ORgc);

		XDrawString(XtDisplay(Stub), XtWindow(ConStub), ORgc,
			 LoggerXoffset, LoggerYdelta,
			 Logger_Msg, strlen(Logger_Msg));
	}

	if(Xfer_Message != NULL_MSG) {

		gcv.foreground = RedPixel;
		XChangeGC(XtDisplay(Stub), ORgc, GCForeground, &gcv);
		(void)Install_Font(TINY, ORgc);

		XDrawString(XtDisplay(Stub), XtWindow(ConStub), ORgc,
			 XferXoffset, XferYdelta, Xfer_Msgs[Xfer_Message],
			 strlen(Xfer_Msgs[Xfer_Message]));
	}

} /* Call_Repaint_Clock */

void Install_Font(int FontSize, GC gc)
{
	extern XGCValues gcv;
	extern XFontStruct *Lft, *Sft, *Tft;
	extern Widget Stub;

	if(FontSize == SMALL) {
		gcv.font = Sft->fid;
		XChangeGC(XtDisplay(Stub), gc, GCFont, &gcv);
	}
	else if(FontSize == LARGE) {
		gcv.font = Lft->fid;
		XChangeGC(XtDisplay(Stub), gc, GCFont, &gcv);
	}
	else {
		gcv.font = Tft->fid;
		XChangeGC(XtDisplay(Stub), gc, GCFont, &gcv);
	}
} /* Install_Font */

void Update_Display_Logger()
{
	extern XGCValues gcv;
	extern Widget Stub, ConStub;
	extern Pixel RedPixel, GreenPixel;
	extern GC ORgc, XORgc;

	gcv.foreground = RedPixel;
	XChangeGC(XtDisplay(Stub), XORgc, GCForeground, &gcv);

	(void)Install_Font(TINY, XORgc);
	XDrawString(XtDisplay(Stub), XtWindow(ConStub), XORgc,
		 LoggerXoffset, LoggerYdelta, Logger_Msg, strlen(Logger_Msg));

	gcv.foreground = GreenPixel;
	XChangeGC(XtDisplay(Stub), XORgc, GCForeground, &gcv);

	XSync(XtDisplay(Stub), False);

} /* Update_Display_Logger */
	
void Update_Display_Xfer(enum xfer_msgs Message)
{
	extern XGCValues gcv;
	extern Widget Stub, ConStub;
	extern Pixel RedPixel, GreenPixel;
	extern GC ORgc, XORgc;

	gcv.foreground = RedPixel;
	XChangeGC(XtDisplay(Stub), XORgc, GCForeground, &gcv);

	(void)Install_Font(TINY, XORgc);
	XDrawString(XtDisplay(Stub), XtWindow(ConStub), XORgc,
		 XferXoffset, XferYdelta, Xfer_Msgs[Message],
		 strlen(Xfer_Msgs[Message]));

	gcv.foreground = GreenPixel;
	XChangeGC(XtDisplay(Stub), XORgc, GCForeground, &gcv);
	XSync(XtDisplay(Stub), False);

} /* Update_Display_Xfer */

