
/* -----                                                                */
/*                                                                      */
/* UWDIALER Version 1.1                                        03/01/95 */
/*                                                                      */
/* AUTHORS: William C. Brown and Gerry A. Graesser                      */
/* VERSION: 1.1 (beta)                                                  */
/* DESCRIPTION: Functions to edit personal systems settings.            */
/*									*/
/*                                                                ----- */

#include <stdio.h>
#include <pwd.h>
#include <termio.h>
#include <unistd.h>
#include <errno.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/OpenLook.h>
#include <Xol/RubberTile.h>
#include <Xol/Form.h>
#include <Xol/FButtons.h>
#include <Xol/Caption.h>
#include <Xol/TextField.h>
#include <Xol/StaticText.h>
#include <Xol/ScrollingL.h>

#include "uwdialer.h"

void Modify_Systems(OlListToken Token, int Indicator)
{
	char Buffer[BUFSIZ];
	char System_Name[BUFSIZ];
	char Temp[BUFSIZ];
	char Target_File[32];
	Boolean Found = False;
	struct sys *ptr;
	FILE *fp, *target, *fopen();
	extern pid_t Uid;
	extern char Personal_File[];
	extern struct sys *systems;
	extern OlListToken LastToken;

	for(ptr=systems;ptr!=NULL;ptr=ptr->next) {
		if(ptr->token == Token) {
			Found = True;
			break;
		}
	}

	if(Found == False) {
		(void)DialerError(NONFATAL, BADSYSTEMSLIST);
		return;
	}

	if((fp = fopen(Personal_File, READ)) == NULL) { 
		(void)DialerError(NONFATAL, PERSONALFILE, Personal_File);
		return;
	}

	/* Make target file semi-random to avoid collisions */
	(void)sprintf(Target_File, "/tmp/.uwdialer.%d", Uid);

	if((target = fopen(Target_File, WRITE)) == NULL) { 
		(void)DialerError(NONFATAL, TARGETFILE);
		(void)fclose(fp);
		return;
	}

	while((fgets(Buffer, BUFSIZ, fp)) != NULL) {
		(void)strcpy(Temp, Buffer);
		(void)strcpy(System_Name, strtok(Buffer, ";"));
		if(strncmp(System_Name, ptr->System, strlen(System_Name))==0) {
			if(Indicator == DELETE)
				continue;
			else if (Indicator == MODIFY) {
				(void)Insert_Modified_System(target);
				continue;
			}
		}
		if(fputs(Temp, target) == EOF) {
			(void)DialerError(NONFATAL, WRITETARGET);
			(void)fclose(fp);
			(void)fclose(target);
			return;
		}
	} /* while */

	(void)fclose(fp);
	(void)fclose(target);

	if((fp = fopen(Personal_File, WRITE)) == NULL) { 
		(void)DialerError(NONFATAL, PERSONALFILE, Personal_File);
		return;
	}

	if((target = fopen(Target_File, READ)) == NULL) { 
		(void)DialerError(NONFATAL, TARGETFILE);
		(void)fclose(fp);
		return;
	}

	while((fgets(Buffer, BUFSIZ, target)) != NULL) {
		if(fputs(Buffer, fp) == EOF) {
		     (void)DialerError(NONFATAL, WRITEPERSONAL, Personal_File);
		     break;
		}
	} /* while */

	(void)fclose(fp);
	(void)fclose(target);
	(void)unlink(Target_File);

	LastToken = NULL;

} /* Modify_Systems */

void Edit_System_List(OlListToken Token)
{
	Boolean Found = False;
	char Buffer[BUFSIZ];
	char System_Name[BUFSIZ];
	char Temp[BUFSIZ];
	FILE *fp, *fopen();
	struct sys *ptr;
	extern Dial_Info DialInfoBuf;
	extern struct sys *systems;
	extern char Personal_File[];

	for(ptr=systems;ptr!=NULL;ptr=ptr->next) {
                if(ptr->token == Token) {
                        Found = True;
                        break;
                }
        }

        if(Found == False) {
                (void)DialerError(NONFATAL, BADSYSTEMSLIST);
                return;
        }

        if((fp = fopen(Personal_File, READ)) == NULL) {
                (void)DialerError(NONFATAL, PERSONALFILE, Personal_File);
                return;
        }

	while((fgets(Buffer, BUFSIZ, fp)) != NULL) {
                (void)strcpy(Temp, Buffer);
                (void)strcpy(System_Name, strtok(Buffer, ";"));
                if(strncmp(System_Name, ptr->System, strlen(System_Name)) == 0)
			break;
        } /* while */

	Temp[strlen(Temp)-1] = NULL;

	(void)fclose(fp);

	(void)Settings(MODIFY);
	(void)Set_Settings_Screen(Temp);
} /* Edit_System_List */

void Insert_Modified_System(FILE *fp)
{
	extern Dial_Info DialInfoBuf;

	(void)fprintf(fp, "%s;", DialInfoBuf.System);
        (void)fprintf(fp, "%s;", DialInfoBuf.Line);
        (void)fprintf(fp, "%d;", DialInfoBuf.Speed);
        switch (DialInfoBuf.CharSize) {
                case CS8:
                        (void)fprintf(fp, "%s;", "CS8");
                        break;
                case CS7:
                        (void)fprintf(fp, "%s;", "CS7");
                        break;
                case CS6:
                        (void)fprintf(fp, "%s;", "CS6");
                        break;
                default:
                        (void)fprintf(fp, "%s;", "CS8");
        }
        switch(DialInfoBuf.Parity) {
                case (PARENB|PARODD):
                        (void)fprintf(fp, "%s;", "ODD");
                        break;
                case (PARENB):
                        (void)fprintf(fp, "%s;", "EVEN");
                        break;
                default:
                        (void)fprintf(fp, "%s;", "NONE");
        }

        switch(DialInfoBuf.Flow) {
                case (IXON|IXOFF):
                        (void)fprintf(fp, "%s\n", "ON");
                        break;
                default:
                        (void)fprintf(fp, "%s\n", "OFF");
        }
} /* Insert_Modified_System */

