
/* -----                                                                */
/*                                                                      */
/* UWDIALER Version 1.1                                        03/01/95 */
/*                                                                      */
/* AUTHORS: William C. Brown and Gerry A. Graesser                      */
/* VERSION: 1.1 (beta)                                                  */
/* DESCRIPTION: This little section of code sets up the dial() function */
/*		and then sets the line terminal disciplines.		*/ 
/*                                                                      */
/*                                                                ----- */

#include <stdio.h>
#include <termio.h>
#include <sys/types.h>
#include <sys/signal.h>
#include <sys/time.h>
#include <dial.h>
#include <stropts.h>
#include <errno.h>
#include <cs.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/OpenLook.h>
#include <Xol/ScrollingL.h>

#include "uwdialer.h"

int pipe_fd[2];
pid_t Server_Pid = 0;
CALL call;
XtInputId Pipe_Output;

extern void Reap_Child();

void Init_Dial(Dial_Info *Dfb, int Ind)
{
	char *Ret;
	char Temp[BUFSIZ];
	char Proto_Buf[10];
	struct sigaction sig;
	extern XtAppContext app_con;
	extern char InitModemLine[];
	extern pid_t Lock_Pid;
	extern int Modem_fd;
	extern time_t Connect_Time;

	(void)memset(&call, NULL, sizeof(CALL));

	(void)Set_Buttons(SET_QUIT, True);
	if(Ind != MODEM) {
		(void)Set_Labels(CONNECT_SET);
		(void)Set_Buttons(SET_FOCUS, False);
		(void)Set_Buttons(SET_LOG, False);
	}
	(void)Set_Buttons(SET_HANGUP, False);

	/* -----					*/
	/* Get the System information.			*/
	/*					  ----- */

	if(Ind == SYSTEMS) Dfb->Line = NULL;

	if(Dfb->Line != NULL) {
		if(Ind != MODEM)
			strcpy(InitModemLine, Dfb->Line);
	} else {
		InitModemLine[0] = 0;
	}

	call.attr  = NULL;
	call.baud  = NULL;
	call.speed = Dfb->Speed;
	call.line  = Dfb->Line;
	call.telno = Dfb->System;

	if(strlen(Dfb->System) != 0) 
		(void)sprintf(Temp, "Dialing %s", Dfb->System);
	else if(strlen(Dfb->Line) != 0)
		if(Ind == MODEM)
			(void)sprintf(Temp, "Initializing %s", Dfb->Line);
		else
			(void)sprintf(Temp, "Accessing %s", Dfb->Line);
	else {
		(void)DialerError(NONFATAL, MISSINGSYSTEM);
		return;
	}

	(void)Display_Msg(False, Temp);

	/* Save the current time for call record */
	if(Ind == MODEM)
		Connect_Time = 0;
	else
		Connect_Time = time(NULL);

	/* -----					*/
	/* Call the remote system.			*/
	/*					  ----- */


	if(pipe(pipe_fd) != 0) {
		(void)DialerError(NONFATAL, OPENPIPEFAILED);
		return;
	}

	/* -----					*/
	/* Run the dial function as a separate process  */
	/* so that X doesn't have to wait for it. This  */
	/* keeps our screens nice an fresh. Return the  */
	/* FD through a streams pipe.			*/
	/*					  ----- */

	/*
	 * Save the current (parent) PID so that we don't
	 * have to take any chances with calling getppid()
	 * in the child process and accidentally ending up
	 * with init's PID (1).
	 */

	Lock_Pid = getpid();

	if((Server_Pid = fork()) == 0) {
		(void)sleep(1);
		(void)close(pipe_fd[0]);
		Modem_fd = dial(call);

		if(Modem_fd < 0) {
			Call_Record(Modem_fd);
			Ret = Resolve_Dial_Error(Modem_fd);
			(void)DialerError(NONFATAL, DIALFAILS, errno, Ret);
			(void)strcpy(Proto_Buf, "Negative");
		} else {
			if(Ind == MODEM)
			{
				struct termios t;

				/*
				 * Hangup NOW since the modem initialization
				 * script may end with "\m\c" which causes
				 * the modem control "lead" to be dropped.
				 */
				tcgetattr(Modem_fd, &t);
				t.c_cflag = 0;
				t.c_lflag = 0;
				t.c_iflag = 0;
				t.c_oflag = 0;
				tcsetattr(Modem_fd, TCSANOW, &t);
				undial(Modem_fd);
				close(Modem_fd);
				Modem_fd = -1;

				/* Make sure lock has been deleted */
				(void)Delete_Lock(DELETE, getpid(), Lock_Pid);
				
				(void)strcpy(Proto_Buf, "Complete");
			} else {
				(void)strcpy(Proto_Buf, "Positive");
			}
		}

		if(write(pipe_fd[1], Proto_Buf, 8) != 8) {
			(void)DialerError(NONFATAL, PROTOBUFWRFAIL);
			(void)exit(0);
		}

		if(Modem_fd >= 0) {
			if(ioctl(pipe_fd[1], I_SENDFD, Modem_fd) < 0) {
				(void)DialerError(NONFATAL, PROTOIOCTL);
				(void)exit(0);
			}
			(void)Delete_Lock(REWRITE, getpid(), Lock_Pid);
		}
		exit(0);
	} /* fork */

	sig.sa_handler = Reap_Child;
        sig.sa_flags = 0;
        sig.sa_flags |= SA_RESTART;
        sigaction(SIGCHLD, &sig, NULL);

	Pipe_Output = XtAppAddInput(app_con, pipe_fd[0], XtInputReadMask,
			Complete_Connection, Dfb);

} /* Init_Dial */

void Chars_At_Modem(XtPointer ClientData, int *Src, XtInputId *inid)
{
        char buffer[BUFSIZ];
	char *ptr;
        int nbytes;
	extern FILE *Logger_fp;
	extern char Logger_Char;
	extern int LOGGER;
	extern pid_t Xfer_Pid;
	extern int Modem_fd;

	if(Xfer_Pid != 0)
		return;

        if((nbytes = read(Modem_fd, buffer, BUFSIZ)) < 0) {
		if(errno == EINTR)
			return;
		(void)DialerError(NONFATAL, ZEROREAD);
	}
        else
                (void)write(fileno(stdout), buffer, nbytes);

	if((LOGGER == ON) && (nbytes > 0))
	{
		int	i, j;
		char	*p;

		/* if there was a leftover '\r' then process it first */
		if(Logger_Char) {
			if(buffer[0] != '\n')
				(void)write(fileno(Logger_fp), "\r", 1);
			Logger_Char = 0;
		}

		i = 0;
		j = nbytes - 1;		/* process last char outside of loop */
		p = &buffer[0];
		while(j != 0) {
			buffer[i] = *p++;
			if((buffer[i] != '\r') || (*p != '\n')) ++i;
			--j;
		}

		/* process last character in buffer */
		if(*p != '\r') {
			buffer[i++] = *p;
		} else {
			Logger_Char = *p;	/* save for next time */
		}

                (void)write(fileno(Logger_fp), buffer, i);
	}

} /* Chars_At_Modem */

char *Resolve_Dial_Error(int Modem_fd)
{
	static char Error[BUFSIZ];

	switch (Modem_fd) {
		case -1:
			(void)strcpy(Error, "interrupt occurred");
			return(Error);
		case -2:
			(void)strcpy(Error, "dialer failed");
			return(Error);
		case -3:
			(void)strcpy(Error, "no answer");
			return(Error);
		case -4:
			(void)strcpy(Error, "illegal baud rate");
			return(Error);
		case -5:
			(void)strcpy(Error, "acu problem");
			return(Error);
		case -6:
			(void)strcpy(Error, "line problem (open failed)");
			return(Error);
		case -7:
			(void)strcpy(Error, "can't open devices file");
			return(Error);
		case -8:
			(void)strcpy(Error, "requested device not available");
			return(Error);
		case -9:
			(void)strcpy(Error, "requested device not known");
			return(Error);
		case -10:
			(void)strcpy(Error, "no devices available at baud rate");
			return(Error);
		case -11:
			(void)strcpy(Error, "no devices known at baud rate");
			return(Error);
		case -12:
			(void)strcpy(Error, "system not in system file");
			return(Error);
		case -13:
			(void)strcpy(Error, "no connection server");
			return(Error);
		case -14:
			return(Error);
		default:
			(void)strcpy(Error, "unknown error");
			return(Error);
	} /* case */
} /* Resolve_Dial_Error */

int Report_Speed(struct termios *t)
{
	if(cfgetospeed(t) == B300) {
		return(300);
	}
	if(cfgetospeed(t) == B1200) {
		return(1200);
	}
	if(cfgetospeed(t) == B2400) {
		return(2400);
	}
	if(cfgetospeed(t) == B4800) {
		return(4800);
	}
	if(cfgetospeed(t) == B9600) {
		return(9600);
	}
	if(cfgetospeed(t) == B19200) {
		return(19200);
	}
	if(cfgetospeed(t) == B38400) {
		return(38400);
	}
	return(0);
} /* Report_Speed */

void Complete_Connection(Dial_Info *Dfb, int *Src, XtInputId *Inid)
{
	extern char *ttyname();
	char buf[BUFSIZ];
	char Temp[BUFSIZ];
	char *Tty;
	struct termios t;
	struct strrecvfd recfd;
	extern Widget LoggerTop;
	extern XtInputId Modem_Input;
	extern XtAppContext app_con;
	extern char InitModemLine[];
	extern pid_t Lock_Pid;
	extern int Modem_fd;
	extern time_t Connect_Time;

	/* -----					*/
	/* Make doublely sure that the lock file has	*/
	/* our PID (Lock_Pid) and not the child's PID	*/
	/* (Server_Pid) as soon as possible (ASAP) so	*/
	/* that uucp or cu can't steal our tty.		*/
	/*					  ----- */

	(void)Delete_Lock(REWRITE, Server_Pid, Lock_Pid);

	(void)memset(buf, NULL, BUFSIZ);
	XtRemoveInput(Pipe_Output);
	Server_Pid = 0;

	if(read(pipe_fd[0], buf, 8) < 8) {
		(void)DialerError(NONFATAL, SERVERTERM); 
		(void)strcpy(buf, "Negative");
	}

	if(strncmp(buf, "Positive", 8) == 0) {
		if(ioctl(pipe_fd[0], I_RECVFD, &recfd) < 0) {
			(void)DialerError(NONFATAL, PROTOIOCTL);
			Modem_fd = -1;
		} else {
			Modem_fd = recfd.fd;
		}
	} else {
		Connect_Time = 0;
		Modem_fd = -1;
	}

        if(Modem_fd < 0) {
                (void)Call_Hangup(NULL, NULL, NULL);
                (void)Call_Repaint_Display(NULL, NULL, NULL);
                return;
        }

	/* Record Time of Connection for the Call Record */
	Connect_Time = time(NULL);

	(void)tcgetattr(Modem_fd, &t);

        t.c_oflag = 0;
        t.c_iflag =  IGNBRK | ISTRIP | IGNPAR | Dfb->Flow;
        t.c_cflag |= CREAD | HUPCL | CLOCAL | Dfb->CharSize | Dfb->Parity;
	t.c_lflag = 0;

	switch (Dfb->Speed) {
		case 300:
			cfsetispeed(&t, B300);
			cfsetospeed(&t, B300);
			break;
		case 1200:
			cfsetispeed(&t, B1200);
			cfsetospeed(&t, B1200);
			break;
		case 2400:
			cfsetispeed(&t, B2400);
			cfsetospeed(&t, B2400);
			break;
		case 4800:
			cfsetispeed(&t, B4800);
			cfsetospeed(&t, B4800);
			break;
		case 9600:
			cfsetispeed(&t, B9600);
			cfsetospeed(&t, B9600);
			break;
		case 19200:
			cfsetispeed(&t, B19200);
			cfsetospeed(&t, B19200);
			break;
		case 38400:
			cfsetispeed(&t, B38400);
			cfsetospeed(&t, B38400);
			break;
	} /* switch */

	if(strlen(call.telno) != 0) {
		(void)sprintf(Temp, "Connected %s - %d",
			 Dfb->System, Report_Speed(&t));
	} else if(strlen(call.line) != 0)
		(void)sprintf(Temp, "Connected %s", Dfb->Line);

        (void)tcsetattr(Modem_fd, TCSANOW, &t);

	Tty = ttyname(Modem_fd);
	if(Tty == NULL) {
		(void)DialerError(NONFATAL, NOTTYNAME);
	} else {
		if(strncmp(Tty, "/dev/", 5) == 0) Tty += 5;
		(void)strcpy(InitModemLine, Tty);
	}

	(void)Set_Buttons(SET_QUIT, True);
	(void)Set_Buttons(SET_SYSTEMS, True);
	(void)Set_Buttons(SET_MANUAL, True);
	(void)Set_Buttons(SET_MODEM, True);
	(void)Set_Buttons(SET_FOCUS, False);
	(void)Set_Buttons(SET_HANGUP, False);
	(void)Set_Buttons(SET_BREAK, False);
	if (LoggerTop == NULL) (void)Set_Buttons(SET_LOG, False);
	(void)Set_Buttons(SET_UPLOAD, False);
	(void)Set_Buttons(SET_DOWNLOAD, False);
	(void)Call_Repaint_Display(NULL, NULL, NULL);
	(void)Update_Connect_Clock((XtPointer)START, NULL);
	(void)Display_Msg(False, Temp);

    	Modem_Input = XtAppAddInput(app_con, Modem_fd, XtInputReadMask,
			 Chars_At_Modem, NULL);

	(void)close(pipe_fd[0]);
	(void)close(pipe_fd[1]);

} /* Complete_Connection */

