
/* -----                                                                */
/*                                                                      */
/* UWDIALER Version 1.1                                        03/01/95 */
/*                                                                      */
/* AUTHORS: William C. Brown and Gerry A. Graesser                      */
/* VERSION: 1.1 (beta)                                                  */
/* DESCRIPTION: Functions for logging screen output from remote system	*/
/*									*/
/*                                                                ----- */

#include <stdio.h>
#include <termios.h>
#include <dial.h>
#include <errno.h>
#include <unistd.h>
#include <sys/param.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/Caption.h>
#include <Xol/OpenLook.h>
#include <Xol/Form.h>
#include <Xol/FButtons.h>
#include <Xol/TextField.h>
#include <Xol/StaticText.h>
#include <Xol/ScrollingL.h>

#include "uwdialer.h"

void Set_Logger(Widget, XtPointer, XtPointer);

static String   On_Fields[] = {XtNlabel, XtNselectProc, XtNclientData};
static XtArgVal On_Items[] =
         {(XtArgVal)"Start logger",  (XtArgVal)Set_Logger, (XtArgVal)ON};

static String   Off_Fields[] = {XtNlabel, XtNselectProc, XtNclientData};
static XtArgVal Off_Items[] =
         {(XtArgVal)"Stop logger",  (XtArgVal)Set_Logger, (XtArgVal)OFF};

char LogFile[MAXPATHLEN];

Widget LoggerTop, LoggerText;

FILE *Logger_fp = NULL;
char Logger_Char = 0;

void Logger_Popup()
{
	int AC = 0;
	int len;
	Arg args[20];
	Widget Form, Cap1, Bu1, Bu2, Invis;
	extern Widget TopLevel;

	(void)Set_Buttons(SET_LOG, True);

	AC = 0;
        XtSetArg(args[AC], XtNtitle, "Logger Popup");   AC++;
        XtSetArg(args[AC], XtNresizeCorners, FALSE);    AC++;
        LoggerTop = XtAppCreateShell("uwdialer", "UWdialer",
                        applicationShellWidgetClass, XtDisplay(TopLevel),
                        args, AC);
	AC = 0;
        Form = XtCreateManagedWidget ("form", formWidgetClass,
                                        LoggerTop, NULL, 0);

	AC = 0;
        XtSetArg(args[AC], XtNlabel, "Log file:");  AC++;
        XtSetArg(args[AC], XtNyRefName, "form");    AC++;
        XtSetArg(args[AC], XtNyOffset, 15);         AC++;
        XtSetArg(args[AC], XtNxOffset, 15);         AC++;
        XtSetArg(args[AC], XtNxAttachRight, TRUE);  AC++;
        XtSetArg(args[AC], XtNxVaryOffset, TRUE);   AC++;
        XtSetArg(args[AC], XtNxAttachOffset, 15);   AC++;
        Cap1 = XtCreateManagedWidget ("cap1", captionWidgetClass,
                                        Form, args, AC);

	AC = 0;
        XtSetArg(args[AC], XtNmaximumSize, 64);    AC++;
        XtSetArg(args[AC], XtNcharsVisible, 64);   AC++;
        LoggerText = XtCreateManagedWidget ("te", textFieldWidgetClass,
                                        Cap1, args, AC);

	len = strlen(LogFile);
	XtSetArg(args[0], XtNstring, LogFile);
	XtSetValues(LoggerText, args, 1);
	XtSetArg(args[0], XtNpreselect, False);
	XtSetValues(LoggerText, args, 1);
	XtSetArg(args[0], XtNcursorPosition, len);
	XtSetValues(LoggerText, args, 1);

        AC = 0;
        XtSetArg(args[AC], XtNbuttonType, OL_OBLONG_BTN);           AC++;
        XtSetArg(args[AC], XtNitemFields, On_Fields);               AC++;
        XtSetArg(args[AC], XtNnumItemFields, XtNumber(On_Fields));  AC++;
        XtSetArg(args[AC], XtNclientData, LoggerTop);               AC++;
        XtSetArg(args[AC], XtNitems, On_Items);                     AC++;
        XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);            AC++;
        XtSetArg(args[AC], XtNmeasure, 1);                          AC++;
        XtSetArg(args[AC], XtNnumItems, 1);                         AC++;
        XtSetArg(args[AC], XtNyRefWidget, Cap1);                    AC++;
        XtSetArg(args[AC], XtNyOffset, 15);                         AC++;
	XtSetArg(args[AC], XtNyAddHeight, True);		    AC++;
        XtSetArg(args[AC], XtNxRefName, "form");                    AC++;
        XtSetArg(args[AC], XtNxOffset, 50);                         AC++;
        Bu1 = XtCreateManagedWidget ("bu", flatButtonsWidgetClass,
                                     Form, args, AC);

        AC = 0;
        XtSetArg(args[AC], XtNbuttonType, OL_OBLONG_BTN);           AC++;
        XtSetArg(args[AC], XtNitemFields, Off_Fields);              AC++;
        XtSetArg(args[AC], XtNnumItemFields, XtNumber(Off_Fields)); AC++;
        XtSetArg(args[AC], XtNclientData, LoggerTop);               AC++;
        XtSetArg(args[AC], XtNitems, Off_Items);                    AC++;
        XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);            AC++;
        XtSetArg(args[AC], XtNmeasure, 1);                          AC++;
        XtSetArg(args[AC], XtNnumItems, 1);                         AC++;
        XtSetArg(args[AC], XtNyRefWidget, Bu1);                     AC++;
        XtSetArg(args[AC], XtNyOffset, 0);                          AC++;
        XtSetArg(args[AC], XtNxRefName, "form");                    AC++;
        XtSetArg(args[AC], XtNxAttachRight, TRUE);                  AC++;
        XtSetArg(args[AC], XtNxVaryOffset, TRUE);                   AC++;
        XtSetArg(args[AC], XtNxAttachOffset, 50);                   AC++;
        Bu2 = XtCreateManagedWidget ("bu", flatButtonsWidgetClass,
                                     Form, args, AC);

	AC = 0;
        XtSetArg(args[AC], XtNstring, "   ");    AC++;
        XtSetArg(args[AC], XtNyRefWidget, Bu2);  AC++;
        XtSetArg(args[AC], XtNyOffset, 0);       AC++;
        XtSetArg(args[AC], XtNyAddHeight, TRUE); AC++;
        Invis = XtCreateManagedWidget ("se", staticTextWidgetClass,
                                        Form, args, AC);

	XtRealizeWidget(LoggerTop);

} /* Logger_Popup */

void Set_Logger(Widget W, XtPointer ClientData, XtPointer CallData)
{
	Arg args[1];
	int rc;
	char *File_Name;
	char TmpFile[MAXPATHLEN];
	extern char CurrentDir[];
	extern int LOGGER;
	extern int Xtermid;
	extern pid_t Gid, Uid;


	XtSetArg(args[0], XtNstring, &File_Name);
	XtGetValues(LoggerText, args, 1);

	if((int)ClientData == ON) {
		if(strlen(File_Name) == 0) {
			(void)DialerError(NONFATAL, NOLOGGERFILE);
			return;
		}
		if(ExpandPath(File_Name, TmpFile) != 0) {
			/* error was printed from ExpandPath() */
			return;
		}
		File_Name = TmpFile;
		/*
		 * Check acces permissions since this program is setgid UUCP
		 */
		rc = access(File_Name, W_OK);
		if (rc != 0)
		{
			char *p;
			char tmpdir[MAXPATHLEN];

			if (errno != ENOENT) {
				(void)DialerError(NONFATAL, LOGGEROPENFAIL, File_Name);
				return;
			}

			p = strrchr(File_Name, '/');
			if (p == NULL) {
				strcpy(tmpdir, CurrentDir);
				if(tmpdir[0] == 0) {
					(void)strcpy(tmpdir, ".");
				}
			} else {
				*p = '\0';
				(void)strcpy(tmpdir, File_Name);
				*p = '/';
				if(tmpdir[0] == 0) {
					(void)DialerError(NONFATAL, NOACCESSDIR, "/");
					return;
				}
			}
			if(access(tmpdir, (R_OK|W_OK|X_OK)) != 0) {
				(void)DialerError(NONFATAL, NOACCESSDIR, tmpdir);
				return;
			}
		}
		if(Logger_fp != NULL) {
			LOGGER = (int)ClientData;
			(void)DialerError(NONFATAL, LOGALREADYON);
		} else {
			if((Logger_fp = fopen(File_Name, APPEND)) == NULL) {
				(void)DialerError(NONFATAL, LOGGEROPENFAIL, File_Name);
				return;
			}
			Logger_Char = 0;
			(void)chown(File_Name, Uid, Gid);
			LOGGER = (int)ClientData;
			(void)Update_Display_Logger();
			(void)strcpy(LogFile, File_Name);
		}
	} else if((int)ClientData == OFF) {
		LOGGER = (int)ClientData;
		if(Logger_fp != NULL) {
			if(Logger_Char) {
				(void)write(fileno(Logger_fp), "\n", 1);
				Logger_Char = 0;
			}
			(void)fclose(Logger_fp);
			Logger_fp = NULL;
			(void)Update_Display_Logger();
		}
	}

	XtDestroyWidget(LoggerTop);
	LoggerTop = NULL;
	(void)Set_Buttons(SET_LOG, False);
	(void)Call_Enter(NULL, (XtPointer)Xtermid, NULL, NULL);
} /* Set_Logger */
