
/* -----                                                                */
/*                                                                      */
/* UWDIALER Version 1.1                                        03/01/95 */
/*                                                                      */
/* AUTHORS: William C. Brown and Gerry A. Graesser                      */
/* VERSION: 1.1 (beta)                                                  */
/* DESCRIPTION: Functions to read and/or update list of systems.	*/
/*									*/
/*                                                                ----- */

#include <stdio.h>
#include <pwd.h>
#include <termio.h>
#include <unistd.h>
#include <sys/param.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/OpenLook.h>
#include <Xol/RubberTile.h>
#include <Xol/Form.h>
#include <Xol/FButtons.h>
#include <Xol/Caption.h>
#include <Xol/TextField.h>
#include <Xol/StaticText.h>
#include <Xol/ScrollingL.h>

#include "uwdialer.h"

static String   Sys_Fields[] = {XtNlabel, XtNselectProc, XtNclientData};
static XtArgVal Sys_Items[] =
         {(XtArgVal)"Personal",  (XtArgVal)Set_Systems, (XtArgVal)PERSONAL,
          (XtArgVal)"System", (XtArgVal)Set_Systems, (XtArgVal)SYSTEM};

static String   Di_Fields[] = {XtNlabel, XtNselectProc};
static XtArgVal Di_Items[] = {(XtArgVal)"Dial",  (XtArgVal)Sys_Dial};

static String   Dis_Fields[] = {XtNlabel, XtNselectProc};
static XtArgVal Dis_Items[] = {(XtArgVal)"Dismiss",  (XtArgVal)Systems_Dismiss};

static String   Add_Fields[] = {XtNlabel, XtNselectProc, XtNbusy};
static XtArgVal Add_Items[] = {(XtArgVal)"Add New System", 
			       (XtArgVal)Call_Add,	
			       (XtArgVal)False,
			       (XtArgVal)"Delete System", 
			       (XtArgVal)Call_Delete,	
			       (XtArgVal)False,
			       (XtArgVal)"Modify System", 
			       (XtArgVal)Call_Edit,	
			       (XtArgVal)False};

#define	NUM_SYS	2
#define	NUM_ADD	3

OlListToken	LastToken;
OlListToken	(*AddItem)();
void		(*TouchItem)(),
		(*UpdateView)(),
		(*DelItem)();

void	Init_List(int);
void	Set_State();
void Fill_System_Buffer(struct sys **, int, char *);
void InsertItem(char *, struct sys **, Boolean, int);

int Current_List = PERSONAL;
struct sys *systems;
Boolean NewList = True;
Boolean Systems_Mapped = False;
char Personal_File[MAXPATHLEN];
char Systems_File[MAXPATHLEN];

Widget Slist, SystemsTop, Add;

void Call_Systems(Widget W, XtPointer ClientData, XtPointer CallData)
{
	Arg args[15];
	int AC;
	Boolean argok;

	Widget  Form,
		Cap1,
		Invis,
		Dismiss,
		Dial,
		Sel;

	extern Widget TopLevel;

	(void)Set_Buttons(SET_SYSTEMS, True);
	(void)Set_Buttons(SET_MANUAL, True);
	(void)Set_Buttons(SET_MODEM, True);

        AC = 0;
        XtSetArg(args[AC], XtNtitle, "Systems Popup");   AC++;
        XtSetArg(args[AC], XtNresizeCorners, FALSE);     AC++;
        SystemsTop = XtAppCreateShell("uwdialer", "UWdialer",
                        applicationShellWidgetClass, XtDisplay(TopLevel),
                        args, AC);

	AC = 0;
        Form = XtCreateManagedWidget ("form", formWidgetClass,
                                      SystemsTop, NULL, 0);

        AC = 0;
        XtSetArg(args[AC], XtNlabel, "System List:"); AC++;
        XtSetArg(args[AC], XtNyRefName, "form");      AC++;
        XtSetArg(args[AC], XtNyOffset, 15);           AC++;
        XtSetArg(args[AC], XtNxOffset, 35);           AC++;
        XtSetArg(args[AC], XtNxAttachRight, TRUE);    AC++;
        XtSetArg(args[AC], XtNxVaryOffset, TRUE);     AC++;
        XtSetArg(args[AC], XtNxAttachOffset, 35);     AC++;
        Cap1 = XtCreateManagedWidget ("cap1", captionWidgetClass,
                                        Form, args, AC);

        AC = 0;
        XtSetArg(args[AC], XtNexclusives, True);                      AC++;
        XtSetArg(args[AC], XtNbuttonType, OL_RECT_BTN);               AC++;
        XtSetArg(args[AC], XtNitemFields, Sys_Fields);                AC++;
        XtSetArg(args[AC], XtNnumItemFields, XtNumber(Sys_Fields));   AC++;
        XtSetArg(args[AC], XtNitems, Sys_Items);                      AC++;
        XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);              AC++;
        XtSetArg(args[AC], XtNmeasure, 1);                            AC++;
        XtSetArg(args[AC], XtNnumItems, NUM_SYS);                     AC++;
        Sel = XtCreateManagedWidget ("excl", flatButtonsWidgetClass,
                                     Cap1, args, AC);

	AC = 0;
	XtSetArg(args[AC], XtNset, True);	AC++;
	OlFlatSetValues(Sel, 0, args, AC);

	AC = 0;
	XtSetArg(args[AC], XtNselectable, True);	AC++;
	XtSetArg(args[AC], XtNviewHeight, 6);		AC++;
	XtSetArg(args[AC], XtNfontColor,
                 CONVERT(Form, "black", "Pixel", 0, &argok));   AC++;
        XtSetArg(args[AC], XtNyRefName, "cap1");        AC++;
        XtSetArg(args[AC], XtNyOffset, 15);             AC++;
        XtSetArg(args[AC], XtNxOffset, 40);             AC++;
        XtSetArg(args[AC], XtNxAttachRight, True);      AC++;
        XtSetArg(args[AC], XtNxVaryOffset, True);       AC++;
        XtSetArg(args[AC], XtNxAttachOffset, 40);       AC++;
        XtSetArg(args[AC], XtNyAddHeight, True);        AC++;
	Slist = XtCreateManagedWidget("slist", scrollingListWidgetClass,
					Form, args, AC);

        AC = 0;
        XtSetArg(args[AC], XtNbuttonType, OL_OBLONG_BTN);           AC++;
        XtSetArg(args[AC], XtNitemFields, Add_Fields);              AC++;
        XtSetArg(args[AC], XtNnumItemFields, XtNumber(Add_Fields)); AC++;
        XtSetArg(args[AC], XtNitems, Add_Items);                    AC++;
        XtSetArg(args[AC], XtNlayoutType, OL_FIXEDCOLS);            AC++;
        XtSetArg(args[AC], XtNmeasure, 1);                          AC++;
        XtSetArg(args[AC], XtNnumItems, NUM_ADD);                   AC++;
        XtSetArg(args[AC], XtNyRefWidget, Slist);                   AC++;
        XtSetArg(args[AC], XtNyOffset, 20);                         AC++;
        XtSetArg(args[AC], XtNyAddHeight, True);                    AC++;
        XtSetArg(args[AC], XtNxRefName, "form");                    AC++;
        XtSetArg(args[AC], XtNxOffset, 75);                         AC++;
        Add = XtCreateManagedWidget ("bu", flatButtonsWidgetClass,
                                     Form, args, AC);

        AC = 0;
        XtSetArg(args[AC], XtNbuttonType, OL_OBLONG_BTN);          AC++;
        XtSetArg(args[AC], XtNitemFields, Di_Fields);              AC++;
        XtSetArg(args[AC], XtNnumItemFields, XtNumber(Di_Fields)); AC++;
        XtSetArg(args[AC], XtNitems, Di_Items);                    AC++;
        XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);           AC++;
        XtSetArg(args[AC], XtNmeasure, 1);                         AC++;
        XtSetArg(args[AC], XtNnumItems, 1);                        AC++;
        XtSetArg(args[AC], XtNyRefWidget, Add);                    AC++;
        XtSetArg(args[AC], XtNyOffset, 20);                        AC++;
        XtSetArg(args[AC], XtNyAddHeight, True);                   AC++;
        XtSetArg(args[AC], XtNxRefName, "form");                   AC++;
        XtSetArg(args[AC], XtNxOffset, 35);                        AC++;
        Dial = XtCreateManagedWidget ("bu", flatButtonsWidgetClass,
                                     Form, args, AC);

        AC = 0;
        XtSetArg(args[AC], XtNbuttonType, OL_OBLONG_BTN);           AC++;
        XtSetArg(args[AC], XtNitemFields, Dis_Fields);              AC++;
        XtSetArg(args[AC], XtNnumItemFields, XtNumber(Dis_Fields)); AC++;
        XtSetArg(args[AC], XtNclientData, SystemsTop);              AC++;
        XtSetArg(args[AC], XtNitems, Dis_Items);                    AC++;
        XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);            AC++;
        XtSetArg(args[AC], XtNmeasure, 1);                          AC++;
        XtSetArg(args[AC], XtNnumItems, 1);                         AC++;
        XtSetArg(args[AC], XtNyRefWidget, Dial);                    AC++;
        XtSetArg(args[AC], XtNyOffset, 0);                          AC++;
        XtSetArg(args[AC], XtNxRefName, "form");                    AC++;
        XtSetArg(args[AC], XtNxAttachRight, TRUE);                  AC++;
        XtSetArg(args[AC], XtNxVaryOffset, TRUE);                   AC++;
        XtSetArg(args[AC], XtNxAttachOffset, 35);                   AC++;
        Dismiss = XtCreateManagedWidget ("bu", flatButtonsWidgetClass,
                                     Form, args, AC);

        AC = 0;
        XtSetArg(args[AC], XtNstring, "   ");        AC++;
        XtSetArg(args[AC], XtNyRefWidget, Dismiss);  AC++;
        XtSetArg(args[AC], XtNyOffset, 0);           AC++;
        XtSetArg(args[AC], XtNyAddHeight, TRUE);     AC++;
        Invis = XtCreateManagedWidget ("se", staticTextWidgetClass,
                                        Form, args, AC);

	AC = 0;
	XtSetArg(args[AC], XtNapplAddItem, 	&AddItem);	AC++;
	XtSetArg(args[AC], XtNapplTouchItem, 	&TouchItem);	AC++;
	XtSetArg(args[AC], XtNapplUpdateView, 	&UpdateView);	AC++;
	XtSetArg(args[AC], XtNapplDeleteItem, 	&DelItem);	AC++;
	XtGetValues(Slist, args, AC);
	XtAddCallback(Slist, XtNuserMakeCurrent, Set_State, NULL);

	(void)Init_List(PERSONAL);

	XtRealizeWidget(SystemsTop);

	Systems_Mapped = True;

} /* Call_Systems */

void Init_List(Ind)
{
	int count = 0;
	int AC;
	char Buffer[BUFSIZ];
	Arg args[2];
	FILE *fp, *fopen();
	Boolean First = True;
	struct sys *ptr;
	struct passwd *pwd, *getpwuid();

	/* -----					*/
	/* Get the systems list started.		*/
	/*					  ----- */

	if(NewList == False) {
		(*UpdateView)(Slist, False);
		(void)DeleteItems(systems);
		(*UpdateView)(Slist, True);
		(void)MyFree(systems);
	}

	if((systems = (struct sys *)calloc(1, sizeof(struct sys))) == NULL) 
		DialerError(FATAL, OUTOFMEMORY);
	ptr = systems;

	if(Ind == PERSONAL) {
		if((fp = fopen(Personal_File, READ)) == NULL) {
			(void)DialerError(NONFATAL, PERSONALFILE, Personal_File);
			return;
		}
	}
	else {
		if((fp = fopen(Systems_File, "r")) == NULL) {
			(void)DialerError(NONFATAL, SYSTEMFILE);
			return;
		}
	}

	while(fgets(Buffer, BUFSIZ, fp) != NULL) {
		if((*Buffer == COMMENT) || (*Buffer == SPACE) ||
		   (*Buffer == NEWLINE) || (*Buffer == TAB))
			continue;
		Buffer[strlen(Buffer)-1] = NULL;
		(void)InsertItem(Buffer, &ptr, First, Ind);
		First = False;
		count++;
	}

	AC = 0;
	if(count > 10)
		XtSetArg(args[AC], XtNviewHeight, 10);		AC++;
	XtSetValues(Slist, args, AC);

	NewList = False;
	(void)fclose(fp);

} /* Init_List */

void InsertItem(char *Buffer, struct sys **Sptr, Boolean First, int Ind)
{
	OlListItem item;
	char *Filler_Entry = "                    ";
	OlListToken Filler_Token;
	short count = 0;

	(*UpdateView)(Slist, False);

	/* -----					*/
	/* This next little gizmo is used to force the  */
	/* scrolling list into a fixed size, larger     */
	/* than the largest system name. 		*/
	/*					  ----- */

	item.label_type = OL_STRING;
	item.attr = count;
	item.label = Filler_Entry;
	item.mnemonic = NULL;
	count++;
	Filler_Token = (*AddItem)(Slist, 0, 0, item);
	(void)(*DelItem)(Slist, Filler_Token);

	/* End of kludge */

	if(First) {
		(void)Fill_System_Buffer(Sptr, Ind, Buffer);
	}
	else {
	    if(((*Sptr)->next = (struct sys *)calloc(1,sizeof(struct sys)))==NULL) 
	   	  DialerError(FATAL, OUTOFMEMORY);
	    (*Sptr) = (*Sptr)->next;
	    (*Sptr)->next = NULL;
	    (void)Fill_System_Buffer(Sptr, Ind, Buffer);
	}

	item.label_type = OL_STRING;
	item.attr = count;
	item.label = (*Sptr)->System;
	item.mnemonic = NULL;
	count++;
	(*Sptr)->token = (*AddItem)(Slist, 0, 0, item);
	(*UpdateView)(Slist, True);

} /* InsertItem */
		

void Fill_System_Buffer(struct sys **Sptr, int Ind, char *Buffer)
{
	void Pad();

	if(Ind == PERSONAL) {
		(void)strcpy((*Sptr)->System,   strtok(Buffer, ";"));
		/*(void)Pad((*Sptr)->System);*/
		(void)strcpy((*Sptr)->Phone,    strtok(NULL, ";"));
		(void)strcpy((*Sptr)->Speed,    strtok(NULL, ";"));
		(void)strcpy((*Sptr)->CharSize, strtok(NULL, ";"));
		(void)strcpy((*Sptr)->Parity,   strtok(NULL, ";"));
		(void)strcpy((*Sptr)->Flow,     strtok(NULL, ";"));
	}
	else
		(void)strcpy((*Sptr)->System, strtok(Buffer, " "));
} /* Fill_System_Buffer */

void Pad(char *string)
{
	char Temp[BUFSIZ];

	(void)strcpy(Temp, string);
	(void)memset(string, NULL, BUFSIZ);
	(void)memset(string, ' ', PADDING);
	(void)strncpy(string, Temp, strlen(Temp));
} /* Pad */

void Set_State(Widget W, XtPointer ClientData, XtPointer CallData)
{
	OlListToken token = (OlListToken)CallData;
	OlListItem *newitem = OlListItemPointer(token);
	OlListItem *lastitem;

	if(LastToken == token) {   /* Unset the current choice */
		newitem->attr &= ~OL_LIST_ATTR_CURRENT;
		(*TouchItem)(Slist, token);
		LastToken = NULL;
		return;
	}

	if(LastToken) {
		lastitem = OlListItemPointer(LastToken);
		if(lastitem->attr & OL_LIST_ATTR_CURRENT)
			lastitem->attr &= ~OL_LIST_ATTR_CURRENT;
		(*TouchItem)(Slist, LastToken);
	}

	newitem->attr |= OL_LIST_ATTR_CURRENT;
	(*TouchItem)(Slist, token);
	LastToken = token;
} /* Set_State */

void Set_Systems(Widget W, XtPointer ClientData, XtPointer CallData)
{
	(void)Init_List((int)ClientData);
	Current_List = (int)ClientData;

	if((int)ClientData == PERSONAL) {
		(void)Set_System_Buttons(False);
	} else {
		(void)Set_System_Buttons(True);
	}
	LastToken = NULL;
} /* Set_Systems */

void Set_System_Buttons(int indicator)
{
	int Sentinel;
	Arg args[1];

	XtSetArg(args[0], XtNbusy, indicator);
	for(Sentinel=0;Sentinel<NUM_ADD;Sentinel++) {
		OlFlatSetValues(Add, Sentinel, args, 1);
	}
} /* Set_System_Buttons */

void Sys_Dial(Widget W, XtPointer ClientData, XtPointer CallData)
{
	extern int Modem_fd;

	if(Modem_fd != 0) {
		(void)DialerError(NONFATAL, ALREADYCONNECT);
		return;
	}

	if(LastToken == NULL) {
		(void)DialerError(NONFATAL, NOITEMSELECTED);
		return;
	}
	
	(void)Construct_Dial_Struct(LastToken);

} /* Sys_Dial */

void Systems_Dismiss(Widget W, XtPointer ClientData, XtPointer CallData)
{
	(void)Systems_Destroy(W, ClientData, CallData);
	(void)Set_Buttons(SET_SYSTEMS, False);
	(void)Set_Buttons(SET_MANUAL, False);
	if(InitModemKey) (void)Set_Buttons(SET_MODEM, False);
} /* Systems_Dismiss */

void Systems_Destroy(Widget W, XtPointer ClientData, XtPointer CallData)
{
	extern pid_t Server_Pid;
	extern int Xtermid;

	LastToken = NULL;
	XtDestroyWidget(ClientData);
	Systems_Mapped = False;

	if(Server_Pid) (void)Call_Enter(NULL, (XtPointer)Xtermid, NULL, NULL);

} /* Systems_Destroy */

void MyFree(struct sys *systems)
{
	if(systems == NULL)
		return;

	(void)MyFree(systems->next);
	(void)free(systems);
} /* MyFree */

void DeleteItems(struct sys *systems)
{
	if((systems == NULL) || (systems->token == NULL))
		return;

	(void)DeleteItems(systems->next);
	(void)(*DelItem)(Slist, systems->token);
}

void Construct_Dial_Struct(OlListToken token)
{
	struct sys *ptr;
	extern Dial_Info DialInfoBuf;
	extern Widget Stub;

	(void)memset(&DialInfoBuf, NULL, sizeof(Dial_Info));

	for(ptr=systems;ptr!=NULL;ptr=ptr->next) {
		if(ptr->token == token)
			break;
	}

	if(Current_List == PERSONAL) {
		DialInfoBuf.Speed    = SetSpeed(ptr);
		DialInfoBuf.CharSize = SetCharSize(ptr);
		DialInfoBuf.Flow     = SetFlow(ptr);
		DialInfoBuf.Parity   = SetParity(ptr);
		DialInfoBuf.System   = ptr->Phone;
	}
	else {  /* UUCP entry */
		DialInfoBuf.Speed    = -1;
		DialInfoBuf.System   = ptr->System;
	}
	
	DialInfoBuf.Line     = NULL;

	XSync(XtDisplay(Stub), False);
	(void)Init_Dial(&DialInfoBuf, SYSTEMS);
	(void)Systems_Destroy(NULL, (XtPointer)SystemsTop, NULL);
		
} /* Construct_Dial_Struct */

int SetParity(struct sys *ptr)
{
	if(strcmp(ptr->Parity, "EVEN") == 0)
		return(PARENB);
	if(strcmp(ptr->Parity, "ODD") == 0)
		return(PARENB | PARODD);

	return(0);
}

int SetFlow(struct sys *ptr)
{
	if(strcmp(ptr->Flow, "ON") == 0)
		return((IXON|IXOFF));
	
	return(0);
}

int SetCharSize(struct sys *ptr)
{
	if(strcmp(ptr->CharSize, "CS8") == 0)
		return(CS8);
	if(strcmp(ptr->CharSize, "CS7") == 0)
		return(CS8);
	if(strcmp(ptr->CharSize, "CS6") == 0)
		return(CS8);

	return(0);
} /* SetCharSize */

int SetSpeed(struct sys *ptr)
{
	if(strcmp(ptr->Speed, "38400") == 0)
		return(38400);
	if(strcmp(ptr->Speed, "19200") == 0)
		return(19200);
	if(strcmp(ptr->Speed, "9600") == 0)
		return(9600);
	if(strcmp(ptr->Speed, "4800") == 0)
		return(4800);
	if(strcmp(ptr->Speed, "2400") == 0)
		return(2400);
	if(strcmp(ptr->Speed, "1200") == 0)
		return(1200);
	if(strcmp(ptr->Speed, "300") == 0)
		return(300);

	return(0);
} /* SetSpeed */

void Call_Add(Widget W, XtPointer ClientData, XtPointer CallData)
{
	XtSetSensitive(SystemsTop, False);
	(void)Settings(SYSTEMS);
} /* Call_Add */

void Call_Delete(Widget W, XtPointer ClientData, XtPointer CallData)
{
	if(LastToken == NULL) {
		(void)DialerError(NONFATAL, NOITEMSELECTED);
		return;
	}

	(void)Modify_Systems(LastToken, DELETE);
	(void)Set_Systems(NULL, (XtPointer)PERSONAL, NULL);
} /* Call_Delete */

void Call_Edit(Widget W, XtPointer ClientData, XtPointer CallData)
{

	if(LastToken == NULL) {
		(void)DialerError(NONFATAL, NOITEMSELECTED);
		return;
	}

	if(access(Personal_File, F_OK) == -1) {
		(void)DialerError(NONFATAL, PERSONALFILE, Personal_File);
		return;
	}

	XtSetSensitive(SystemsTop, False);
	(void)Edit_System_List(LastToken);
} /* Call_Edit */

