
/* -----								*/
/* 									*/
/* UWDIALER Version 1.1					       03/01/95 */
/*									*/
/* AUTHORS: William C. Brown and Gerry A. Graesser			*/
/* VERSION: 1.1 (beta)							*/
/* DESCRIPTION: To provide a nice X based dialer interface that works	*/
/*		in conjunction with the connection server in SVR4.2.	*/
/*									*/
/*								  ----- */

#include <stdio.h>
#include <stdlib.h>
#include <stropts.h>
#include <termio.h>
#include <pwd.h>
#include <grp.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/OpenLook.h>
#include <Xol/RubberTile.h>
#include <Xol/ControlAre.h>
#include <Xol/Stub.h>
#include <Xol/StaticText.h>
#include <Xol/FButtons.h>
#include <Xol/ScrollingL.h>

#ifdef	SYSLOG
#include <syslog.h>
#endif

#define	Version	"UWDIALER Version 1.1"
#include "uwdialer.h"

/* Widgets */

Widget  TopLevel,
	Rubber,
	BaseControl,
	Control1,
	Flats,
	Count,
	ConStub,
	Stub;

/* Some extra globals */

GC ORgc, XORgc;
XFontStruct *Lft, *Sft, *Tft;
XtAppContext app_con;
XtIntervalId TimerId;
XGCValues gcv;

int Xterm_Type = TERMINAL;
int Xtermid;
int LOGGER = OFF;
int SendTo = REMOTECOMMAND;
int Xfer_Direction = 0;

extern int ConStubHeight;	/* See display_update.c */

/*
 * This next little guy tells uwdialer to talk to the modem or
 * to a local ksh session.
 */

int Master_fd = 0;
int Slave_fd = 	0;
int Modem_fd = 	0;

pid_t Uid, Gid;
pid_t Lock_Pid = 0;
pid_t LocalCommandPid = 0;

char *ProgName;
struct termios old;
Pixel RedPixel, GreenPixel;
pid_t Xfer_Pid = 0;
int Xfer_Kill = 0;

char InitModemKey;
char CurrentDir[MAXPATHLEN];
char Logname[MAXLOGNAMELEN];
char HomeDir[MAXPATHLEN];
char DownloadDir[MAXPATHLEN];
char UploadDir[MAXPATHLEN];

XtInputId Term_Input;
XtInputId Modem_Input;
XtInputId Local_Input;

void Call_Enter(Widget, XtPointer, XEvent *, Boolean *);
void Get_Dialer_Options(int, char **);
void Usage();

/*
 * FlatItems stuff
 */

enum button_sets Button_Set = INITIAL_SET;

const enum dial_buttons Button_Sets[BUTTON_SETS][MAIN_BUTTONS] = {

	SET_QUIT, SET_SYSTEMS, SET_MANUAL, SET_HELP, SET_HANGUP, SET_MODEM,	  /* INITIAL_SET */
	SET_FOCUS, SET_HANGUP, SET_BREAK, SET_LOG, SET_UPLOAD, SET_DOWNLOAD,	  /* CONNECT_SET */
	SET_FOCUS, SET_HANGUP, SET_TERMINATE, SET_LOG, SET_UPLOAD, SET_DOWNLOAD}; /* TRANSFER_SET */

const char Button_Label[DIAL_BUTTONS][16] = {
	"Quit",
	"Systems",
	"Manual Dial",
	"Focus",
	"Help",
	"Hangup",
	"Init Modem",
	"Send Break",
	"Log",
	"Upload",
	"Download",
	"Terminate"};

void (*Button_Funcs[DIAL_BUTTONS])() = {
	Call_Quit,
	Call_Systems,
	Call_Manual,
	Call_Focus,
	Call_Help,
	Call_Hangup,
	Call_Modem,
	Call_SndBrk,
	Call_Log,
	Call_Upload,
	Call_Downld,
	Call_Terminate};

/* In But_Item[], make sure the label(s) with the longest names in each column are used */

static String	But_Fields[] = {XtNlabel, XtNselectProc, XtNclientData, XtNbusy, XtNlabelJustify};
static XtArgVal But_Items[] = {
				(XtArgVal)Button_Label[SET_QUIT],
				(XtArgVal)Call_Button,
				(XtArgVal)0,
				(XtArgVal)False,
				(XtArgVal)OL_CENTER,

				(XtArgVal)Button_Label[SET_SYSTEMS],
				(XtArgVal)Call_Button,
				(XtArgVal)1,
				(XtArgVal)False,
				(XtArgVal)OL_CENTER,

				(XtArgVal)Button_Label[SET_MANUAL],
				(XtArgVal)Call_Button,
				(XtArgVal)2,
				(XtArgVal)False,
				(XtArgVal)OL_CENTER,

				(XtArgVal)Button_Label[SET_FOCUS],
				(XtArgVal)Call_Button,
				(XtArgVal)3,
				(XtArgVal)False,
				(XtArgVal)OL_CENTER,

				(XtArgVal)Button_Label[SET_HANGUP],
				(XtArgVal)Call_Button,
				(XtArgVal)4,
				(XtArgVal)False,
				(XtArgVal)OL_CENTER,

				(XtArgVal)Button_Label[SET_MODEM],
				(XtArgVal)Call_Button,
				(XtArgVal)5,
				(XtArgVal)False,
				(XtArgVal)OL_CENTER,
}; /* But_Items[] */

void Call_Button(Widget W, XtPointer ClientData, XtPointer CallData)
{
	enum dial_buttons Button;

	if ((((int)ClientData >= 0) && ((int)ClientData < MAIN_BUTTONS)) &&
	     ((Button_Set >= 0) && (Button_Set < BUTTON_SETS))) {
		Button = Button_Sets[Button_Set][(int)ClientData];
		(void)Button_Funcs[Button](W, ClientData, CallData);
	}
}

void main(int argc, char *argv[])
{
	int AC = 0;
	int X, Y, XtermWidth, FlatWidth, height, b, d;
	unsigned int Num_Cld;
	struct group *grp, *getgrnam();
	static char *Large_Font = "-*-courier-*-*-*-*-24-*-*-*-*-*-*";
	static char *Small_Font = "-*-courier-*-*-*-*-20-*-*-*-*-*-*";
	static char *Tiny_Font = "-*-lucidatypewriter-*-*-*-*-*-80-*-*-*-*-*-*";
	char *L_Font;
	char *S_Font;
	char *T_Font;

	char *Icon_Name;
	Boolean argok;
	Arg args[10];
	Window RootWin, Parent, GParent, *Children;
	XtTranslations Translation;
	Pixmap Prog_Pixmap;
	XrmValue from, to;

	ProgName = argv[0];
	Uid = getuid();
	Gid = getgid();
#ifdef	SYSLOG
	openlog("uwdialer", LOG_PID, LOG_USER);
	syslog(LOG_INFO, "%s:user=%s\n", Version, cuserid(NULL));
#endif

	OlToolkitInitialize(&argc, argv, (XtPointer)NULL);

	(void)Get_Dialer_Options(argc, argv);

	XtSetArg(args[AC], XtNresizeCorners, FALSE);           AC++;
	XtSetArg(args[AC], XtNtitle, Version);                 AC++;
	XtSetArg(args[AC], XtNallowShellResize, True);	       AC++;
	TopLevel = XtAppInitialize(&app_con, "uwdialer", "UWdialer", 0,
				   &argc, argv, NULL, args, AC);

	/* -----					*/
	/* Get the pixmap name from the Xresource db    */
	/* and convert that into something we can use.	*/
	/*					  ----- */

	Icon_Name = XGetDefault(XtDisplay(TopLevel), argv[0], "iconPixmap");

	from.addr = Icon_Name;
	from.size = strlen(Icon_Name)+1;
	to.addr   = (char *)&Prog_Pixmap;
	to.size   = sizeof(Pixmap);

	if(XtConvertAndStore(TopLevel, XtRString, &from,
			     XtRPixmap, &to) != False) {
		XtSetArg(args[0], XtNiconPixmap, Prog_Pixmap);
		XtSetValues(TopLevel, args, 1);
	}

	/* -----				*/
	/* Get the fonts names out of the       */
	/* resource file. If not there default  */
	/* to some hand selected fonts.		*/
	/*				  ----- */

	L_Font = XGetDefault(XtDisplay(TopLevel), argv[0], "largeFont");
	S_Font = XGetDefault(XtDisplay(TopLevel), argv[0], "smallFont");
	T_Font = XGetDefault(XtDisplay(TopLevel), argv[0], "tinyFont");

	if(L_Font == NULL)
		L_Font = Large_Font;

	if(S_Font == NULL)
		S_Font = Small_Font;

	if(T_Font == NULL)
		T_Font = Tiny_Font;

	/* -----				*/
	/* Create a base level rubber tile.	*/
	/*				  ----- */

	AC = 0;
	XtSetArg(args[AC], XtNshadowThickness, 2);             AC++;
	XtSetArg(args[AC], XtNshadowType, OL_SHADOW_OUT);      AC++;
	XtSetArg(args[AC], XtNpaneGravity, NorthSouthGravity); AC++;
	Rubber = XtCreateManagedWidget ("rubber", rubberTileWidgetClass,
					TopLevel, args, AC);

	/* -----				*/
	/* Now a control area for the buttons   */
	/* and such, then a stub widget that we */
	/* will use to reparent the Xterm.	*/
	/*				  ----- */

	AC = 0;
	XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);  AC++;
	XtSetArg(args[AC], XtNmeasure, 1); 		  AC++;
	XtSetArg(args[AC], XtNshadowThickness, 0);        AC++;
	XtSetArg(args[AC], XtNshadowType, OL_SHADOW_IN); AC++;
	Control1 = XtCreateManagedWidget ("control", controlAreaWidgetClass,
					Rubber, args, AC);

	AC = 0;
	XtSetArg(args[AC], XtNbuttonType, OL_OBLONG_BTN); 	    AC++;
	XtSetArg(args[AC], XtNitemFields, But_Fields);    	    AC++;
	XtSetArg(args[AC], XtNnumItemFields, XtNumber(But_Fields)); AC++;
	XtSetArg(args[AC], XtNitems, But_Items);                    AC++;
	XtSetArg(args[AC], XtNlayoutType, OL_FIXEDCOLS);            AC++;
	XtSetArg(args[AC], XtNmeasure, BUTTON_COLUMNS);             AC++;
	XtSetArg(args[AC], XtNnumItems, MAIN_BUTTONS);              AC++;
	XtSetArg(args[AC], XtNlayoutHeight, OL_MAXIMIZE);           AC++;
	XtSetArg(args[AC], XtNlayoutWidth, OL_MAXIMIZE);            AC++;
	Flats = XtCreateManagedWidget("button", flatButtonsWidgetClass,
				     Control1, args, AC);
 
	AC = 0;
	XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);  AC++;
	XtSetArg(args[AC], XtNmeasure, 1); 		  AC++;
	XtSetArg(args[AC], XtNshadowThickness, 1);        AC++;
	XtSetArg(args[AC], XtNshadowType, OL_SHADOW_IN);  AC++;
	Count = XtCreateManagedWidget("count", controlAreaWidgetClass,
					Control1, args, AC);

	AC = 0;
	XtSetArg(args[AC], XtNheight, ConStubHeight);            AC++;
	XtSetArg(args[AC], XtNwidth, 10);                        AC++;
	XtSetArg(args[AC], XtNbackground,
		 CONVERT(Count, "black", "Pixel", 0, &argok));   AC++;
	XtSetArg(args[AC], XtNborderWidth, 1);                   AC++;
	XtSetArg(args[AC], XtNborderColor,
		 CONVERT(Count, "green", "Pixel", 0, &argok));   AC++;
	XtSetArg(args[AC], XtNexpose, Call_Repaint_Display);     AC++;
	ConStub = XtCreateManagedWidget ("stub", stubWidgetClass,
					Count, args, AC);

	AC = 0;
	XtSetArg(args[AC], XtNheight, 10);                AC++;
	XtSetArg(args[AC], XtNwidth, 10);                 AC++;
	XtSetArg(args[AC], XtNweight, 1);                 AC++;
	Stub = XtCreateManagedWidget ("stub", stubWidgetClass,
					Rubber, args, AC);

	/* -----					*/
	/* Realize the widget tree, then do some size   */
	/* calculations.				*/
	/*					  ----- */

	XtRealizeWidget (TopLevel);

	Xtermid = atoi(getenv("WINDOWID"));
	XGetGeometry(XtDisplay(Stub), Xtermid, &RootWin, &X, &Y,
		     &XtermWidth, &height, &b, &d);
 
	if(Xterm_Type == TERMINAL) {
		XQueryTree(XtDisplay(Stub), Xtermid, &RootWin,
			   &Parent, &Children, &Num_Cld);
		XQueryTree(XtDisplay(Stub), Parent, &RootWin,
			   &GParent, &Children, &Num_Cld);
		XReparentWindow(XtDisplay(Stub), GParent, XtWindow(Stub), 0, 0);
	}
	else if (Xterm_Type == XTERM) {
    		XReparentWindow(XtDisplay(Stub), Xtermid, XtWindow(Stub), 0, 0);
	}
	/*
	XtAddEventHandler(Stub, EnterWindowMask, True, Call_Enter, Xtermid);
	*/
	AC = 0;
	XtSetArg(args[AC], XtNheight, height);                AC++;
	if(Xterm_Type == TERMINAL) {
		XtSetArg(args[AC], XtNwidth, XtermWidth+20);  AC++;
	}
	else {
		XtSetArg(args[AC], XtNwidth, XtermWidth);     AC++;
	}
	XtSetValues(Stub, args, AC);

	XGetGeometry(XtDisplay(Stub), XtWindow(Flats), &RootWin, &X, &Y,
		     &FlatWidth, &height, &b, &d);

	if(Xterm_Type == XTERM)
		FlatWidth = FlatWidth + 20;

	AC = 0;
	XtSetArg(args[AC], XtNwidth, XtermWidth-FlatWidth); AC++; 
	XtSetValues(ConStub, args, AC);

	/* -----						*/
	/* Get a GC for use with the status display.		*/
	/*						  ----- */

	RedPixel       = GetPixelByName(ConStub, "red");
	GreenPixel     = GetPixelByName(ConStub, "green");
	gcv.foreground = GreenPixel;
	gcv.background = GetPixelByName(ConStub, "black");
	gcv.function   = GXxor;

	if ((Lft = XLoadQueryFont(XtDisplay(Stub), L_Font)) != NULL)
		gcv.font = Lft->fid; 
	else
		(void)DialerError(FATAL, FONTALLOCATION, Large_Font);

	if ((Sft = XLoadQueryFont(XtDisplay(Stub), S_Font)) == NULL)
		(void)DialerError(FATAL, FONTALLOCATION, Small_Font);

	if ((Tft = XLoadQueryFont(XtDisplay(Stub), T_Font)) == NULL)
		(void)DialerError(FATAL, FONTALLOCATION, Tiny_Font);

	XORgc = XCreateGC(XtDisplay(Stub), XtWindow(ConStub),
			GCForeground | GCBackground | GCFont | GCFunction,
			&gcv);

	gcv.function   = GXor;
	ORgc = XCreateGC(XtDisplay(Stub), XtWindow(ConStub),
			GCForeground | GCBackground | GCFont | GCFunction,
			&gcv);

	/* -----						*/
	/* Get everything initialized.				*/
	/*						  ----- */

	(void)Init_Personal_File();

	(void)Set_Buttons(SET_QUIT, False);
	(void)Set_Buttons(SET_SYSTEMS, False);
	(void)Set_Buttons(SET_MANUAL, False);
	(void)Set_Buttons(SET_FOCUS, True);
	(void)Set_Buttons(SET_HELP, False);
	(void)Set_Buttons(SET_HANGUP, True);
	(void)Set_Buttons(SET_MODEM, True);
	(void)Set_Buttons(SET_BREAK, True);
	(void)Set_Buttons(SET_LOG, True);
	(void)Set_Buttons(SET_UPLOAD, True);
	(void)Set_Buttons(SET_DOWNLOAD, True);
	(void)Set_Buttons(SET_TERMINATE, True);
	(void)Set_Labels(INITIAL_SET);
	(void)Update_Connect_Clock((XtPointer)INIT, NULL);
	(void)Display_Msg(True, "Initializing...");
	(void)Init_Xterm();
	(void)Init_Paths();
	(void)Init_Proto();
	if(InitModemKey) (void)Set_Buttons(SET_MODEM, False);
	(void)Display_Msg(False, "Ready...");

	XtAppMainLoop(app_con);

} /* main() */

void Call_Enter(Widget W, XtPointer ClientData, XEvent *event, Boolean *Ctn)
{
	XSetInputFocus(XtDisplay(Stub), ClientData, RevertToNone, 0);
}

void Call_Focus(Widget W, XtPointer ClientData, XtPointer CallData)
{
	(void)Call_Enter(NULL, (XtPointer)Xtermid, NULL, NULL);
}

void Get_Dialer_Options(int argc, char *argv[])
{
	int c;
	char *terminal_type;
	extern char *optarg;
	extern int optind;

	terminal_type = "";

	while ((c = getopt(argc, argv, "t:")) != EOF) {
		switch(c) {
			case 't':
				terminal_type = optarg;
				break;
			case '?':
				(void)Usage();
		}
	}

	if(strcmp(terminal_type, "terminal") == 0)
		Xterm_Type = TERMINAL;
	else if(strncmp(terminal_type, "xterm", 5) == 0)
		Xterm_Type = XTERM;
	else
		(void)Usage();
} /* Get_Dialer_Options */

void Usage()
{
	fprintf(stderr, "USAGE: [xterm_program] -e uwdialer -t xterm_type\n");
	(void)exit(0);
} /* Usage */

void Init_Personal_File()
{
	extern char Personal_File[];
	extern char Systems_File[];
	char	Temp_File[MAXPATHLEN];
	char	Comment[80];
	char	*Path;
	struct passwd *pwd, *getpwuid();
	int	Fd, len;

	pwd = getpwuid(getuid());
	if (pwd == NULL) {
		(void)DialerError(NONFATAL, CANTGETPWD, getuid());
		return;
	}

	Path = XGetDefault(XtDisplay(TopLevel), ProgName, "systemsFile");
	if((Path != NULL) && (ExpandPath(Path, Temp_File) == 0)) {
		if(Temp_File[0] == '/') {
			(void)strcpy(Systems_File, Temp_File);
		} else {
			(void)sprintf(Systems_File, "/etc/uucp/%s", Temp_File);
		}
	} else {
		(void)strcpy(Systems_File, "/etc/uucp/Systems");
	}

	Path = XGetDefault(XtDisplay(TopLevel), ProgName, "personalFile");
	if((Path != NULL) && (ExpandPath(Path, Temp_File) == 0)) {
		if(Temp_File[0] == '/') {
			(void)strcpy(Personal_File, Temp_File);
		} else {
			(void)sprintf(Personal_File, "%s/%s", pwd->pw_dir, Temp_File);
		}
	} else {
		(void)sprintf(Personal_File, "%s/%s", pwd->pw_dir, PERSONAL_FILE);
	}

	if((access(Personal_File, F_OK) == -1) && (errno == ENOENT)) {
		Path = strrchr(Personal_File, '/');
		if(Path != NULL) {
			*Path = 0;
		}
		if((Path == NULL) || (Personal_File[0] == 0) ||
		   (access(Personal_File, (R_OK|W_OK|X_OK)) != 0)) {
			(void)DialerError(FATAL, CREATEPERSONAL, Personal_File);
			(void)strcpy(Personal_File, "/dev/null");
		} else {
			*Path = '/';
			(void)sprintf(Comment, "# UWdialer personal systems file.\n");
			len = strlen(Comment);

			Fd = creat(Personal_File, 0600);
			if (Fd < 0) {
				(void)DialerError(FATAL, CREATEPERSONAL, Personal_File);
			} else {
				(void)write(Fd, Comment, len);
				(void)chown(Personal_File, Uid, Gid);
				(void)close(Fd);
			}
		}
	}
	if(access(Personal_File, W_OK) != 0) {
		(void)DialerError(FATAL, NOACCESSPER, Personal_File);
		(void)strcpy(Personal_File, "/dev/null");
	}

} /* Init_Personal_File */
