
/* -----                                                                */
/*                                                                      */
/* UWDIALER Version 1.1                                        03/01/95 */
/*                                                                      */
/* AUTHORS: William C. Brown and Gerry A. Graesser                      */
/* VERSION: 1.1 (beta)                                                  */
/* DESCRIPTION: UWdialer protocol transfer/local command functions.	*/
/*									*/
/*                                                                ----- */

#include <stdio.h>
#include <termios.h>
#include <dial.h>
#include <signal.h>
#include <fcntl.h>
#include <stropts.h>
#include <poll.h>
#include <errno.h>
#include <unistd.h>
#include <sys/param.h>
#include <errno.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/Caption.h>
#include <Xol/OpenLook.h>
#include <Xol/Form.h>
#include <Xol/FButtons.h>
#include <Xol/TextField.h>
#include <Xol/StaticText.h>
#include <Xol/ScrollingL.h>

#include "uwdialer.h"

#define MASTER "/dev/ptmx"

void Reap_Child();

Widget XferTop, XferTe1, XferTe2, XferTe3;

int	Xfer_Protocol;
int	Proto_Measure;
int	Proto_Num;

char	Transmit_Cmd[PROTO_NUM][PROCMDLEN];
char	Receive_Cmd[PROTO_NUM][PROCMDLEN];
char	Proto_Label[PROTO_NUM][PROLABLEN];

static String   Proto_Fields[] = {XtNlabel, XtNselectProc, XtNclientData, XtNlabelJustify};
static XtArgVal Proto_Items[] =
         {(XtArgVal)Proto_Label[0], (XtArgVal)Set_Protocol, (XtArgVal)ASCII+0, OL_CENTER,
          (XtArgVal)Proto_Label[1], (XtArgVal)Set_Protocol, (XtArgVal)ASCII+1, OL_CENTER,
          (XtArgVal)Proto_Label[2], (XtArgVal)Set_Protocol, (XtArgVal)ASCII+2, OL_CENTER,
          (XtArgVal)Proto_Label[3], (XtArgVal)Set_Protocol, (XtArgVal)ASCII+3, OL_CENTER,
          (XtArgVal)Proto_Label[4], (XtArgVal)Set_Protocol, (XtArgVal)ASCII+4, OL_CENTER,
          (XtArgVal)Proto_Label[5], (XtArgVal)Set_Protocol, (XtArgVal)ASCII+5, OL_CENTER,
          (XtArgVal)Proto_Label[6], (XtArgVal)Set_Protocol, (XtArgVal)ASCII+6, OL_CENTER,
          (XtArgVal)Proto_Label[7], (XtArgVal)Set_Protocol, (XtArgVal)ASCII+7, OL_CENTER,
          (XtArgVal)Proto_Label[8], (XtArgVal)Set_Protocol, (XtArgVal)ASCII+8, OL_CENTER,
          (XtArgVal)Proto_Label[9], (XtArgVal)Set_Protocol, (XtArgVal)ASCII+9, OL_CENTER};


static String   Xfer_Fields[] = {XtNlabel, XtNselectProc};
static XtArgVal Xfer_Items[] =
         {(XtArgVal)"Execute",  (XtArgVal)Start_Xfer};

static String   Dismiss_Fields[] = {XtNlabel, XtNselectProc};
static XtArgVal Dismiss_Items[] =
         {(XtArgVal)"Dismiss",  (XtArgVal)Drop_Xfer};

void Xfer_Popup(int Direction)
{
        int AC = 0;
        Arg args[20];
        Widget Form, Cap1, Cap2, Cap3, Cap4, Bu1, Bu2, Invis, Protocol;
        extern Widget TopLevel;
	extern int Modem_fd;
	extern int Xfer_Direction;

	if(Modem_fd == 0) {
		(void)DialerError(NONFATAL, NOTCONNECTED);
		return;
	}

	(void)Set_Buttons(SET_UPLOAD, True);
	(void)Set_Buttons(SET_DOWNLOAD, True);

	Xfer_Direction = Direction;

        AC = 0;
        XtSetArg(args[AC], XtNtitle, "Transfer Popup");   AC++;
        XtSetArg(args[AC], XtNresizeCorners, FALSE);      AC++;
        XferTop = XtAppCreateShell("uwdialer", "UWdialer",
                        applicationShellWidgetClass, XtDisplay(TopLevel),
                        args, AC);
        AC = 0;
        Form = XtCreateManagedWidget ("form", formWidgetClass,
                                      XferTop, NULL, 0);

	AC = 0;
        XtSetArg(args[AC], XtNlabel, "Transfer Protocol:");  AC++;
        XtSetArg(args[AC], XtNyRefWidget, Form);             AC++;
        XtSetArg(args[AC], XtNyOffset, 15);                  AC++;
        XtSetArg(args[AC], XtNxRefWidget, Form);             AC++;
        XtSetArg(args[AC], XtNxOffset, 15);                  AC++;
        Cap1 = XtCreateManagedWidget ("cap1", captionWidgetClass,
                                        Form, args, AC);

	AC = 0;
        XtSetArg(args[AC], XtNbuttonType, OL_RECT_BTN);                AC++;
        XtSetArg(args[AC], XtNexclusives, True);                       AC++;
        XtSetArg(args[AC], XtNnoneSet, True);                          AC++;
        XtSetArg(args[AC], XtNitemFields, Proto_Fields);               AC++;
        XtSetArg(args[AC], XtNnumItemFields, XtNumber(Proto_Fields));  AC++;
        XtSetArg(args[AC], XtNitems, Proto_Items);                     AC++;
        XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);               AC++;
        XtSetArg(args[AC], XtNmeasure, Proto_Measure);                 AC++;
        XtSetArg(args[AC], XtNnumItems, Proto_Num);                    AC++;
        Protocol = XtCreateManagedWidget ("excl", flatButtonsWidgetClass,
                                          Cap1, args, AC);

	AC = 0;
	if(Direction == UPLOAD) {
        	XtSetArg(args[AC], XtNlabel, "Source File:");  AC++;
	}
	else {
        	XtSetArg(args[AC], XtNlabel, "Target File:");  AC++;
	}
        XtSetArg(args[AC], XtNyRefWidget, Cap1); AC++;
        XtSetArg(args[AC], XtNxRefWidget, Form); AC++;
        XtSetArg(args[AC], XtNyAddHeight, True); AC++;
        XtSetArg(args[AC], XtNyOffset, 15);      AC++;
        XtSetArg(args[AC], XtNxOffset, 15);      AC++;
        Cap2 = XtCreateManagedWidget ("cap2", captionWidgetClass,
                                        Form, args, AC);

        AC = 0;
        XtSetArg(args[AC], XtNmaximumSize, 64);    AC++;
        XtSetArg(args[AC], XtNcharsVisible, 64);   AC++;
        XferTe1 = XtCreateManagedWidget ("te", textFieldWidgetClass,
                                        Cap2, args, AC);

	AC = 0;
        XtSetArg(args[AC], XtNlabel, "Directory:  "); AC++;
        XtSetArg(args[AC], XtNyRefWidget, Cap2);  AC++;
        XtSetArg(args[AC], XtNxRefWidget, Form);  AC++;
        XtSetArg(args[AC], XtNyAddHeight, True);  AC++;
        XtSetArg(args[AC], XtNyOffset, 15);       AC++;
        XtSetArg(args[AC], XtNxOffset, 15);       AC++;
        Cap3 = XtCreateManagedWidget ("cap3", captionWidgetClass,
                                        Form, args, AC);

        AC = 0;
        XtSetArg(args[AC], XtNmaximumSize, 64);    AC++;
        XtSetArg(args[AC], XtNcharsVisible, 64);   AC++;
        XferTe2 = XtCreateManagedWidget ("te", textFieldWidgetClass,
                                        Cap3, args, AC);

	AC = 0;
        XtSetArg(args[AC], XtNlabel, "Command:"); AC++;
        XtSetArg(args[AC], XtNyRefWidget, Cap3);  AC++;
        XtSetArg(args[AC], XtNxRefWidget, Form);  AC++;
        XtSetArg(args[AC], XtNyAddHeight, True);  AC++;
        XtSetArg(args[AC], XtNyOffset, 15);       AC++;
        XtSetArg(args[AC], XtNxOffset, 15);       AC++;
        Cap4 = XtCreateManagedWidget ("cap4", captionWidgetClass,
                                        Form, args, AC);

        AC = 0;
        XtSetArg(args[AC], XtNmaximumSize, 64);    AC++;
        XtSetArg(args[AC], XtNcharsVisible, 64);   AC++;
        XferTe3 = XtCreateManagedWidget ("te", textFieldWidgetClass,
                                        Cap4, args, AC);

        AC = 0;
        XtSetArg(args[AC], XtNbuttonType, OL_OBLONG_BTN);             AC++;
        XtSetArg(args[AC], XtNitemFields, Xfer_Fields);               AC++;
        XtSetArg(args[AC], XtNnumItemFields, XtNumber(Xfer_Fields));  AC++;
        XtSetArg(args[AC], XtNclientData, XferTop);                   AC++;
        XtSetArg(args[AC], XtNitems, Xfer_Items);                     AC++;
        XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);              AC++;
        XtSetArg(args[AC], XtNmeasure, 1);                            AC++;
        XtSetArg(args[AC], XtNnumItems, 1);                           AC++;
        XtSetArg(args[AC], XtNyRefWidget, Cap4);                      AC++;
        XtSetArg(args[AC], XtNyOffset, 15);                           AC++;
        XtSetArg(args[AC], XtNyAddHeight, True);                      AC++;
        XtSetArg(args[AC], XtNxRefName, "form");                      AC++;
        XtSetArg(args[AC], XtNxOffset, 50);                           AC++;
        Bu1 = XtCreateManagedWidget ("bu", flatButtonsWidgetClass,
                                     Form, args, AC);

        AC = 0;
        XtSetArg(args[AC], XtNbuttonType, OL_OBLONG_BTN);                AC++;
        XtSetArg(args[AC], XtNitemFields, Dismiss_Fields);               AC++;
        XtSetArg(args[AC], XtNnumItemFields, XtNumber(Dismiss_Fields));  AC++;
        XtSetArg(args[AC], XtNclientData, XferTop);                      AC++;
        XtSetArg(args[AC], XtNitems, Dismiss_Items);                     AC++;
        XtSetArg(args[AC], XtNlayoutType, OL_FIXEDROWS);                 AC++;
        XtSetArg(args[AC], XtNmeasure, 1);                               AC++;
        XtSetArg(args[AC], XtNnumItems, 1);                              AC++;
        XtSetArg(args[AC], XtNyRefWidget, Bu1);                          AC++;
        XtSetArg(args[AC], XtNyOffset, 0);                               AC++;
        XtSetArg(args[AC], XtNxRefName, "form");                         AC++;
        XtSetArg(args[AC], XtNxAttachRight, TRUE);                       AC++;
        XtSetArg(args[AC], XtNxVaryOffset, TRUE);                        AC++;
        XtSetArg(args[AC], XtNxAttachOffset, 50);                        AC++;
        Bu2 = XtCreateManagedWidget ("bu", flatButtonsWidgetClass,
                                     Form, args, AC);
        AC = 0;
        XtSetArg(args[AC], XtNstring, "   ");    AC++;
        XtSetArg(args[AC], XtNyRefWidget, Bu2);  AC++;
        XtSetArg(args[AC], XtNyOffset, 0);       AC++;
        XtSetArg(args[AC], XtNyAddHeight, TRUE); AC++;
        Invis = XtCreateManagedWidget ("se", staticTextWidgetClass,
                                        Form, args, AC);

	XtRealizeWidget(XferTop);
} /* Xfer_Popup */

void Set_Protocol(Widget W, XtPointer ClientData, XtPointer CallData)
{
	Arg args[1];
	char *Cmd, *Dir, *Temp;
	extern Widget TopLevel;
	extern char *ProgName;
	extern int Xfer_Direction;
	extern int Xfer_Protocol;
	extern char DownloadDir[];
	extern char UploadDir[];

	if (((int)ClientData >= ASCII) &&
	    ((int)ClientData < ASCII+PROTO_NUM)) {
		Xfer_Protocol = (int)ClientData;

		if(Xfer_Direction == UPLOAD) {
			Cmd = Transmit_Cmd[Xfer_Protocol-ASCII];
			Dir = UploadDir;
			Temp = "Transmit";
		} else {
			Cmd = Receive_Cmd[Xfer_Protocol-ASCII];
			Dir = DownloadDir;
			Temp = "Receive";
		}
		if (Cmd[0] == 0)
		{
			char	resource[80];

			sprintf(resource, "%s%s", Proto_Label[Xfer_Protocol-ASCII], Temp);
	        	Temp = XGetDefault(XtDisplay(TopLevel), ProgName, resource);
			if (Temp != NULL) {
				(void)strncpy(Cmd, Temp, 64);
			}
		}

		XtSetArg(args[0], XtNstring, Dir);
		XtSetValues(XferTe2, args, 1);

		XtSetArg(args[0], XtNstring, Cmd);
		XtSetValues(XferTe3, args, 1);
	}
} /* Set_Protocol */

void Start_Xfer(Widget W, XtPointer ClientData, XtPointer CallData)
{
	Arg args[1];
	int len, rc;
	char Temp[BUFSIZ];
	char Replace[25];
	char *Cmd, *Command;
	char *Dir, *Directory;
	char *File;
	char *Suffix;
	char *ptr1, *ptr2;
	struct sigaction sig;
	extern int Xfer_Direction;
	extern int Xfer_Kill;
	extern int Xfer_Protocol;
	extern int Modem_fd;
	extern int Xtermid;
	extern pid_t Gid;
	extern pid_t Xfer_Pid;
	extern char CurrentDir[];
	extern char DownloadDir[];
	extern char UploadDir[];

	XtSetArg(args[0], XtNstring, &File);
	XtGetValues(XferTe1, args, 1);
	XtSetArg(args[0], XtNstring, &Directory);
	XtGetValues(XferTe2, args, 1);
	XtSetArg(args[0], XtNstring, &Command);
	XtGetValues(XferTe3, args, 1);

	if(Xfer_Direction == UPLOAD) {
		Dir = UploadDir;
		Cmd = Transmit_Cmd[Xfer_Protocol-ASCII];
		(void)strcpy(Replace, "${Source_File}");
	} else {
		Dir = DownloadDir;
		Cmd = Receive_Cmd[Xfer_Protocol-ASCII];
		(void)strcpy(Replace, "${Target_File}");
	}

	len = strlen(Replace);

	(void)memset(Temp, NULL, BUFSIZ);

	if(strlen(File) == 0) {
		rc = 0;
		for(ptr2=ptr1=Command;ptr2!=NULL;ptr1++) {
			ptr2=strchr(ptr1, '$');
			if(ptr2 != NULL) {
				if(strncasecmp(ptr2, Replace, len) == 0) {
					rc = 1;
					break;
				}
			}
		}
		if ((rc) || (Xfer_Protocol == ASCII)) {
			(void)DialerError(NONFATAL, NOFILESPECIFIED);
			return;
		}
		if(Command != NULL) (void)strcpy(Temp, Command);
	} else {
		rc = 1;
		for(ptr1=Command,ptr2=Temp;*ptr1!=NULL;ptr1++) {
			if((*ptr1 == '$') &&
			   (strncasecmp(ptr1, Replace, len) == 0)) {
				(void)strcat(ptr2, File);
				ptr1 = ptr1 + len - 1;
				ptr2 = ptr2 + strlen(File);
				rc = 0;
				continue;
			}
			*ptr2++ = *ptr1;
		} /* for */
		if (rc) {
			(void)DialerError(NONFATAL, FILESPECIFIED);
			return;
		}
	}

	if(Directory != NULL) {
		if (strcmp(Directory, Dir) != 0)
		{
			char	TmpDir[MAXPATHLEN];

			if(ExpandPath(Directory, TmpDir) != 0) {
				/* error was printed from ExpandPath() */
				return;
			}
			if(access(TmpDir, (R_OK|X_OK)) != 0) {
				(void)DialerError(NONFATAL, NOACCESSDIR, TmpDir);
				return;
			}
			(void)strcpy(Dir, TmpDir);
		}
	} else {
		if(strcmp(Dir, CurrentDir) != 0) {
			/* Upload/Download from the current directory */
			(void)strcpy(Dir, CurrentDir);
		}
	}

	Xfer_Kill = 0;

	if(Xfer_Protocol == ASCII) {

		switch(Xfer_Pid = fork()) {
			extern struct termios Tc_New;
			struct termios new;
		case 0:
			(void)sleep(1);

			/* Suspend Raw Output Mode */
			(void)memcpy(&new, &Tc_New, sizeof(struct termios));
			new.c_oflag |= (OPOST);
			tcsetattr(0, TCSANOW, &new);

			/*
			 * Define a new process control group
			 * so the parent uwdialer process can
			 * terminate any transfer process that
			 * ends up getting created by this process.
			 */
			(void)setpgrp();

			(void)close(0);
			(void)close(1);
			(void)dup(Modem_fd);
			(void)dup(Modem_fd);
			(void)setgid(Gid);
			if(strcmp(CurrentDir, Dir) != 0) {
				if(chdir(Dir) != 0) {
					(void)DialerError(FATAL, CANTCHDIR, Dir);
				}
			}
			if(Xfer_Direction == DOWNLOAD) {
				(void)Receive_Ascii_File(File);
			} else {
				(void)Transmit_Ascii_File(File);
			}
			(void)tcflush(fileno(stdout), TCOFLUSH);
			exit(0);
		case -1:
			(void)DialerError(FATAL, CANTCREATECHILD);
			Xfer_Pid = 0;
			break;
		default:
			(void)Set_Buttons(SET_BREAK, True);
			(void)Set_Buttons(SET_TERMINATE, False);
			(void)Set_Labels(TRANSFER_SET);
			if(Xfer_Direction == UPLOAD)
				Xfer_Message = UPLOAD_MSG;
			else
				Xfer_Message = DOWNLOAD_MSG;
			(void)Update_Display_Xfer(Xfer_Message);
		}

	} else {	/* not ASCII protocol */

		if((Command != NULL) && (*Command)) {
			if (strcmp(Command, Cmd) != 0) {
				/*
				 * save Command line without Source/Target File
				 */
				(void)strcpy(Cmd, Command);
			}

			if(Temp[0] == '!') {
				StartLocalCommand(Temp, Dir);
			} else {
				StartRemoteProtocol(Temp, Dir);
			}
		} else {
			/* clear previous command entry ? */
			if(*Cmd) *Cmd = 0;
		}

	} /* end else Xfer_Protocol == ASCII */

	(void)Drop_Xfer(NULL, (XtPointer)XferTop, NULL);
	(void)Call_Enter(NULL, (XtPointer)Xtermid, NULL, NULL);

} /* Start_Xfer */

void Drop_Xfer(Widget W, XtPointer ClientData, XtPointer CallData)
{
	extern Widget XferTop;
	extern int Xfer_Direction;
	extern pid_t Xfer_Pid;
	extern int Xtermid;
	extern int Modem_fd;
	extern int SendTo;

	XtDestroyWidget(ClientData);
	XferTop = NULL;

	if(Xfer_Pid == 0) {
		Xfer_Direction = 0;
		if(Modem_fd) {
			(void)Set_Buttons(SET_UPLOAD, False);
			(void)Set_Buttons(SET_DOWNLOAD, False);
		}
		if(SendTo == LOCALCOMMAND) {
			(void)Set_Buttons(SET_UPLOAD, True);
			(void)Set_Buttons(SET_DOWNLOAD, True);
		}
	}

	(void)Call_Enter(NULL, (XtPointer)Xtermid, NULL, NULL);
} /* Start_Xfer */

void Reap_Child()
{
	int stat;
	int child_pid;
	extern struct termios Tc_New;
	extern int Modem_fd;
	extern int Xfer_Direction;
	extern pid_t Xfer_Pid;
	extern int Xfer_Kill;
	extern int Xtermid;
	extern Widget Flats;
	extern XtInputId Local_Input;
	extern int SendTo;
	extern int Master_fd;

	child_pid = wait(&stat);

	/*
	 * If the process that terminated was the local command
	 * process, then stop taking input on that fd.
	 */

	if(SendTo == LOCALCOMMAND) {
		XtRemoveInput(Local_Input);
		SendTo = REMOTECOMMAND;
	}

	/*
	 * If transfer was terminated (killed)
	 * then allow a few seconds for any
	 * grandchild[ren] to terminate before
	 * flushing any remaining unread data.
	 */

	if(Xfer_Kill) {
		if(Modem_fd) (void)tcflush(Modem_fd, TCIFLUSH);
		Xfer_Kill = 0;
	}

	/* Resume Raw Output Mode */
	(void)tcsetattr(0, TCSANOW, &Tc_New);

	if(Modem_fd) {
		(void)Set_Buttons(SET_BREAK, False);
		(void)Set_Buttons(SET_UPLOAD, False);
		(void)Set_Buttons(SET_DOWNLOAD, False);
		(void)Set_Buttons(SET_TERMINATE, True);
		(void)Set_Labels(CONNECT_SET);
		OlUpdateDisplay(Flats);
	}

	if(Xfer_Pid != 0)
		(void)Update_Display_Xfer(Xfer_Message);

	Xfer_Message = NULL_MSG;
	Xfer_Direction = 0;
	Xfer_Pid = 0;
} /* Reap_Child() */

void Receive_Ascii_File(char *file)
{
	int fd;
	int pollret, nbytes;
	int count = 0;
	int TimeOut = -1;
	char Buffer[BUFSIZ];
	char Extra_Return = 0;
	char *ptr;
	struct pollfd pfds[1];

	pfds->fd = fileno(stdin);
	pfds->events = POLLIN;

	if((fd = open(file, O_WRONLY|O_CREAT, 0644)) < 0) {
		(void)DialerError(NONFATAL, ASCIIDOWNLOAD, file);
		return;
	}

	while((pollret = poll(pfds, 1, TimeOut)) >= 0) {
		if(pollret == 0) {
			if(count++ == 2)
				break;
		}

		if(pfds->revents & POLLIN) {
			TimeOut = 1000;
			if((nbytes = read(pfds->fd, Buffer, BUFSIZ)) > 0)
			{
				int	i, j;
				char	*p;

				/* if there was a leftover '\r' then process it first */
				if(Extra_Return) {
					if(Buffer[0] != '\n')
						(void)write(fd, "\r", 1);
					Extra_Return = 0;
				}

				i = 0;
				j = nbytes - 1;		/* process last char outside of loop */
				p = &Buffer[0];
				while(j != 0) {
					Buffer[i] = *p++;
					if((Buffer[i] != '\r') || (*p != '\n')) ++i;
					--j;
				}

				/* process last character in buffer */
				if(*p != '\r') {
					Buffer[i++] = *p;
				} else {
					Extra_Return = *p;	/* save for next time */
				}

                		(void)write(fd, Buffer, i);
			}
		}
	}

	if(Extra_Return)
		(void)write(fd, "\n", 1);
	(void)close(fd);
	exit(0);
} /* Receive_Ascii_File */

void Transmit_Ascii_File(char *file)
{
	int fd, nbytes;
	char Buffer[BUFSIZ];

	if((fd = open(file, O_RDONLY)) < 0) {
		(void)DialerError(NONFATAL, ASCIIUPLOAD, file);
		return;
	}

	while((nbytes = read(fd, Buffer, BUFSIZ)) > 0) {
		(void)write(fileno(stdout), Buffer, nbytes);
	}

	(void)tcdrain(fileno(stdout));
	(void)close(fd);

} /* Transmit_Ascii_File */

void StartRemoteProtocol(char *Command, char *Dir)
{
	int    i;
	extern pid_t Gid;
	extern pid_t Xfer_Pid;
	extern int Modem_fd;
	extern int Xfer_Direction;
	extern char CurrentDir[];

	switch(Xfer_Pid = fork()) {
		extern struct termios Tc_New;
		struct termios new;
	case 0:
		(void)sleep(1);

		/* Suspend Raw Output Mode */
		(void)memcpy(&new, &Tc_New, sizeof(struct termios));
		new.c_oflag |= (OPOST);
		tcsetattr(0, TCSANOW, &new);

		/*
		 * Define a new process control group
		 * so the parent uwdialer process can
		 * terminate any transfer process that
		 * ends up getting created by this process.
		 */
		(void)setpgrp();

		/* arrange file descriptors */
		(void)close(0);
		(void)close(1);
		(void)dup(Modem_fd);
		(void)dup(Modem_fd);

		/* cleanup any/all unused file descriptors */
		for(i=3; i<20; i++) {
			(void)close(i);
		}

		/* Reset any special GID privilages */
		(void)setgid(Gid);

		/* Change current working directory if needed */
		if(strcmp(CurrentDir, Dir) != 0) {
			if(chdir(Dir) != 0) {
				(void)DialerError(FATAL, CANTCHDIR, Dir);
			}
		}
		(void)execlp("sh", "sh", "-c", Command, 0);
		(void)DialerError(FATAL, CANTCREATECHILD);
		exit(0);
	case -1:
		(void)DialerError(FATAL, CANTCREATECHILD);
		Xfer_Pid = 0;
		break;
	default:
		(void)Set_Buttons(SET_BREAK, True);
		(void)Set_Buttons(SET_TERMINATE, False);
		(void)Set_Labels(TRANSFER_SET);
		if(Xfer_Direction == UPLOAD)
			Xfer_Message = UPLOAD_MSG;
		else
			Xfer_Message = DOWNLOAD_MSG;
		(void)Update_Display_Xfer(Xfer_Message);
	}
}

void StartLocalCommand(char *Command, char *Dir)
{
	int   i;
	char *slavename;
	struct sigaction sig, osig;
	struct termios new;
	extern int Master_fd;
	extern int Modem_fd;
	extern int Slave_fd;
	extern int SendTo;
	extern struct termios Tc_New;
	extern pid_t Gid;
	extern pid_t Xfer_Pid;
	extern int Xfer_Direction;
	extern XtAppContext app_con;
	extern XtInputId Local_Input;
	extern char CurrentDir[];

	/*
	 * This next line of code directs UWdialer to write stdin
	 * to our local command instead of the modem.
	 */

	SendTo = LOCALCOMMAND;

	/*
	 * Setup now to start our local command
	 */

	if(Master_fd != 0)
		(void)close(Master_fd);

	if((Master_fd = open(MASTER, O_RDWR, 0666)) < 0) {
		(void)DialerError(NONFATAL, NOLOCALTERMINAL);
		Master_fd = 0;
		return;
	}

	slavename = (char *)ptsname(Master_fd);

	/*
	 * temporarily we need to shutoff our signal handler
	 * so that grantpt() can listen for its own child process
	 * coming down
	 */

        sig.sa_handler = SIG_DFL;
        sig.sa_flags = 0;
        sig.sa_flags |= SA_RESTART;
	sigaction(SIGCHLD, &sig, &osig);

	if(grantpt(Master_fd) < 0) {
		fprintf(stderr, "ERRNO = %d\n", errno);
		(void)DialerError(NONFATAL, NOGRANT);
		(void)close(Master_fd);
		Master_fd = 0;
		return;
	}

	if(unlockpt(Master_fd) < 0) {
		close(Master_fd);
		(void)DialerError(NONFATAL, NOUNLOCK);
		(void)close(Master_fd);
		Master_fd = 0;
		return;
	}

	/*
	 * re-enable the signal handler
	 */

	sigaction(SIGCHLD, &osig, NULL);

	/*
	 * Get to the real command
	 */

	if(*Command == '!')
		++Command;

	switch (Xfer_Pid = fork()) {

	case 0:
		/*
		 * Define a new process control group
		 * so the parent uwdialer process can
		 * terminate any transfer process that
		 * ends up getting created by this process.
		 * And to also prevent the user from using
		 * pg(1) in a pipeline which will cause
		 * uwdialer to lock-up.
		 */
		(void)setpgrp();

		if((Slave_fd = open(slavename, O_RDWR, 0600)) < 0) {
			(void)DialerError(NONFATAL, SLAVEOPEN);
			SendTo = REMOTECOMMAND;
			close(Master_fd);
			return;
		}

                if (ioctl(Slave_fd, I_PUSH, "ptem") < 0) {
			(void)DialerError(NONFATAL, IOCTLFAILED);
			SendTo = REMOTECOMMAND;
                        close(Master_fd);
			return;
                }

                if (ioctl(Slave_fd, I_PUSH, "ldterm") < 0) {
			(void)DialerError(NONFATAL, IOCTLFAILED);
			SendTo = REMOTECOMMAND;
                        close(Master_fd);
			return;
                }

		/*
		 * Setup same stty(1) settings as parent xterm
		 */
		(void)tcgetattr(Slave_fd, &new);
		(void)memcpy(&new.c_cc[0], &Tc_New.c_cc[0], sizeof(cc_t)*NCCS);
		cfsetispeed(&new, cfgetispeed(&Tc_New));
		cfsetospeed(&new, cfgetospeed(&Tc_New));
		(void)tcsetattr(Slave_fd, TCSANOW, &new);

		/*
	   	 * Arrange the filedescriptors
		 */

		(void)close(0);
		(void)close(1);
		(void)close(2);
		(void)dup(Slave_fd);
		(void)dup(Slave_fd);
		(void)dup(Slave_fd);
		(void)close(Slave_fd);

		/* Make sure the Modem FD is available on 3 */
		if(Modem_fd != 3) {
			(void)close(3);
			(void)dup(Modem_fd);
		}

		/* Cleanup any/all other FDs which are not needed */
		for(i=4; i<20; i++) {
			(void)close(i);
		}

		/* Reset any special GID privilages */
		(void)setgid(Gid);

		/* Change current working directory if needed */
		if(strcmp(CurrentDir, Dir) != 0) {
			if(chdir(Dir) != 0) {
				(void)DialerError(FATAL, CANTCHDIR, Dir);
			}
		}

		if(*Command == 0)
			execl("/bin/ksh", "ksh", 0);
		else {
			sleep(1);
			execl("/bin/ksh", "ksh", "-c", Command, 0);
		}

		(void)DialerError(FATAL, CANTCREATECHILD);
		exit(0);

	case -1:
		(void)DialerError(FATAL, CANTCREATECHILD);
		SendTo = REMOTECOMMAND;
		(void)close(Master_fd);
		Master_fd = 0;
		Xfer_Pid = 0;
		break;

	default:
		/*
		 * Add an input to the new Master_fd to read from the remote
		 * ksh
		 */
		Local_Input = XtAppAddInput(app_con, Master_fd, XtInputReadMask,
				ReadFromLocalCommand, NULL);
		(void)Set_Buttons(SET_BREAK, True);
		(void)Set_Buttons(SET_TERMINATE, False);
		(void)Set_Labels(TRANSFER_SET);
		if(Xfer_Direction == UPLOAD)
			Xfer_Message = LOCAL_UP_MSG;
		else
			Xfer_Message = LOCAL_DOWN_MSG;
		(void)Update_Display_Xfer(Xfer_Message);
	}

} /* StartLocalCommand */

void ReadFromLocalCommand(XtPointer ClientData, int *Src, XtInputId *inid)
{
	int nbytes;
	char buffer[BUFSIZ];
	extern int Master_fd;

	if((nbytes = read(Master_fd, buffer, BUFSIZ)) < 0) {
		if(errno == EINTR)
			return;
                (void)DialerError(NONFATAL, ZEROREAD);
        }
        else
                (void)write(fileno(stdout), buffer, nbytes);
}

