
/* -----                                                                */
/*                                                                      */
/* UWDIALER Version 1.1                                        03/01/95 */
/*                                                                      */
/* AUTHORS: William C. Brown and Gerry A. Graesser                      */
/* VERSION: 1.1 (beta)                                                  */
/* DESCRIPTION: Enhancement Functions for Transfer Popup		*/
/*									*/
/*                                                                ----- */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stropts.h>
#include <termios.h>
#include <pwd.h>
#include <grp.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <Xol/OpenLook.h>
#include <Xol/RubberTile.h>
#include <Xol/ControlAre.h>
#include <Xol/Stub.h>
#include <Xol/StaticText.h>
#include <Xol/FButtons.h>
#include <Xol/ScrollingL.h>
#include <Xol/TextField.h>

#include "uwdialer.h"


void Init_Proto()
{
	extern	Widget TopLevel;
	extern	char *ProgName;
	extern	int Proto_Measure;
	extern	int Proto_Num;
	char	*Protocols;

	Protocols = XGetDefault(XtDisplay(TopLevel), ProgName, "Protocols");
	if (Protocols != NULL) {
		Proto_Num = sscanf(Protocols, "%11s %11s %11s %11s %11s %11s %11s %11s %11s",
			&Proto_Label[1], &Proto_Label[2], &Proto_Label[3],
			&Proto_Label[4], &Proto_Label[5], &Proto_Label[6],
			&Proto_Label[7], &Proto_Label[8], &Proto_Label[9]);
	}
	if (Proto_Num < PROTO_NUM) {
		++Proto_Num;
	} else {
		Proto_Num = 1;
	}
	if ((Proto_Num >= 5) && (strlen(Protocols) >= 50)) {
		Proto_Measure = 2;
	} else {
		Proto_Measure = 1;
	}
	(void)strcpy(Proto_Label[0], "ASCII");
	(void)strcpy(Transmit_Cmd[0], "internal read < ${Source_File}");
	(void)strcpy(Receive_Cmd[0], "internal function > ${Target_File}");
}

int ExpandPath(char *Path, char Buffer[MAXPATHLEN])
{
	extern	char HomeDir[];
	struct	passwd	*pwd;
	char	*p, *s;
	char	path[MAXPATHLEN];

	path[0] = 0;

	if (Path != NULL) {
		/* Be nice and strip leading/trailing white space */
		(void)sscanf(Path, "%s", path);
	}

	p = path;

	if (*p == '~') {

		++p;

		if (*p == 0) {
			strcpy(Buffer, HomeDir);
		} else if (*p == '/') {
			(void)sprintf(Buffer, "%s%s", HomeDir, p);
		} else {
			s = strrchr(p, '/');
			if (s != NULL) {
				*s++ = 0;
			} else {
				s = NULL;
			}
			pwd = getpwnam(p);

			if (pwd == NULL) {
				(void)DialerError(NONFATAL, NONUSER, p);
				return(1);
			}
			if (s != NULL) {
				sprintf(Buffer, "%s/%s", pwd->pw_dir, s);
			} else {
				strcpy(Buffer, pwd->pw_dir);
			}
		}
	} else {
		strcpy(Buffer, p);
	}

	return(0);
}

void Init_Paths()
{
	extern	char HomeDir[];
	extern	char Logname[];
	extern	char CurrentDir[];
	extern	char DownloadDir[];
	extern	char UploadDir[];
	extern	char LogFile[];
	extern	char ManTtyLine[];
	extern	char InitModemLine[];
	extern	char CallRecordFile[];
	extern	char *ProgName;
	extern	pid_t Uid, Gid;
	extern	Widget TopLevel;
	struct	passwd *pwd;
	struct	stat statbuf;
	int	len;
	char	*Temp;
	char	*Path;
	char	TmpDir[MAXPATHLEN];

	pwd = getpwuid(getuid());
	if (pwd == NULL) {
		(void)DialerError(FATAL, CANTGETPWD, getuid());
	} else {
		(void)sprintf(HomeDir, "%s", pwd->pw_dir);
		(void)sprintf(Logname, "%s", pwd->pw_name);
	}

	CurrentDir[0] = 0;
	Temp = XGetDefault(XtDisplay(TopLevel), ProgName, "startupDirectory");
	if (Temp != NULL) {
		if (ExpandPath(Temp, TmpDir) == 0) {
			if (access(TmpDir, (R_OK|X_OK)) != 0) {
				(void)DialerError(NONFATAL, NOACCESSDIR, TmpDir);
			} else {
				if (chdir(TmpDir) != 0) {
					(void)DialerError(NONFATAL, CANTCHDIR, TmpDir);
				} else {
					(void)strcpy(CurrentDir, TmpDir);
				}
			}
		}
	}
	if (CurrentDir[0] == 0) {
		Path = getcwd(CurrentDir, MAXPATHLEN);
		if (Path == NULL) {
			(void)DialerError(FATAL, CANTGETCWD);
		}
	}

	(void)strcpy(DownloadDir, CurrentDir);	/* default */

	Temp = XGetDefault(XtDisplay(TopLevel), ProgName, "downloadDirectory");
	if (Temp != NULL) {
		if (ExpandPath(Temp, TmpDir) == 0) {
			if (access(TmpDir, (R_OK|X_OK)) != 0) {
				(void)DialerError(NONFATAL, NOACCESSDIR, TmpDir);
			} else {
				(void)strcpy(DownloadDir, TmpDir);
			}
		}
	}

	(void)strcpy(UploadDir, CurrentDir);	/* default */

	Temp = XGetDefault(XtDisplay(TopLevel), ProgName, "uploadDirectory");
	if (Temp != NULL) {
		if (ExpandPath(Temp, TmpDir) == 0) {
			if (access(TmpDir, (R_OK|X_OK)) != 0) {
				(void)DialerError(NONFATAL, NOACCESSDIR, TmpDir);
			} else {
				(void)strcpy(UploadDir, TmpDir);
			}
		}
	}

	Path = CurrentDir;	/* LogFile default */

	Temp = XGetDefault(XtDisplay(TopLevel), ProgName, "logFile");
	if (Temp != NULL) {
		if (ExpandPath(Temp, TmpDir) == 0) {
			Path = TmpDir;
		}
	}
	(void)strcpy(LogFile, Path);
	if ((stat(Path, &statbuf) == 0) &&
	   ((statbuf.st_mode & S_IFMT) == S_IFDIR)) {

		len = strlen(LogFile);
		if ((len) && (LogFile[len-1] != '/')) {
			LogFile[len] = '/';
			LogFile[++len] = '\0';
		}
	}

	Temp = XGetDefault(XtDisplay(TopLevel), ProgName, "manTtyLine");
	if (Temp != NULL) {
		strncpy(ManTtyLine, Temp, TTY_SYS_NAME_MAX);
	}

	Temp = XGetDefault(XtDisplay(TopLevel), ProgName, "initModemKey");
	if (Temp != NULL) {
		len = strlen(Temp);
		if ((len > 1) || ((len == 1) && (isalpha((int)(*Temp))) == 0)) {
			(void)DialerError(NONFATAL, BADMODEMKEY, Temp);
		} else {
			if (*Temp)
				InitModemKey = *Temp;
			else
				InitModemKey = NULLMODEMKEY;
		}
	}

	if ((ManTtyLine[0]) && (InitModemKey)) {
		(void)strcpy(InitModemLine, ManTtyLine);
	}

	Temp = XGetDefault(XtDisplay(TopLevel), ProgName, "callRecord");
	if (Temp != NULL) {
		if (ExpandPath(Temp, TmpDir) == 0) {
			if (TmpDir[0] == '/') {
				(void)strcpy(CallRecordFile, TmpDir);
			} else {
				(void)sprintf(CallRecordFile, "%s/%s", HomeDir, TmpDir);
			}
			if ((access(CallRecordFile, F_OK) == -1) && (errno == ENOENT)) {
				Temp = strrchr(CallRecordFile, '/');
				if (Temp != NULL) *Temp = 0;
				if ((Temp == NULL) ||
				    (access(CallRecordFile, (R_OK|W_OK|X_OK)) != 0)) {
					(void)DialerError(NONFATAL, CREATECALLREC, CallRecordFile);
					CallRecordFile[0] = 0;
				} else {
					int Fd;

					*Temp = '/';
					Fd = creat(CallRecordFile, 0600);
					if (Fd < 0) {
						(void)DialerError(NONFATAL,
							CREATECALLREC, CallRecordFile);
						CallRecordFile[0] = 0;
					} else {
						(void)chown(CallRecordFile, Uid, Gid);
						(void)close(Fd);
					}
				}
			} else if (access(CallRecordFile, W_OK) != 0) {
				(void)DialerError(NONFATAL, NOACCESSCREC, CallRecordFile);
				CallRecordFile[0] = 0;
			}
		}
	}
}

